﻿#-*- coding: utf-8 -*-
import requests
from enigma import gFont, eTimer, ePicLoad, loadPNG, getDesktop, eListboxPythonMultiContent, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_VALIGN_BOTTOM
from enigma import eEPGCache
from Components.config import config
from Components.Label import Label
from Components.ScrollLabel import ScrollLabel
from Components.Sources.StaticText import StaticText
from Components.ActionMap import NumberActionMap, ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch
from Screens.Screen import Screen
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Screens.MessageBox import MessageBox
from Tools.Directories import fileExists, SCOPE_PLUGINS
from Tools import Directories
from Tools.LoadPixmap import LoadPixmap
from twisted.web.client import downloadPage, getPage, error
from twisted.internet import defer
import re, shutil, datetime, time
import os
import sys
import socket
import ssl
import enigma

#from urllib2 import Request, urlopen, URLError
# our custom classes
from .SkySkipWordsSelect import SkySkipWordsSelect
from .SkyRecorderSettings import SkyRecorderSettings
from .SkyTimerRec import SkyTimerRec
from .SkySql import *
from .SkyMainFunctions import *

# TODO Archiv Screen
from .SkyRecorderArchiv import SkyRecorderArchiv

# New Whitelist
from .SkyWhitelist import SkyWhitelist

# SkyTheMovieDB
from .SkyTheMovieDB import SkyTheMovieDB

from .SkyMyColorBackground import MyColorScreen

class SkyRecorderMainScreen(MyColorScreen):

	def __init__(self, session):
		self.session = session
		
		self.agent = b'python-requests/2.25.1'  #getUserAgent()
		self.headers = getHttpHeader()
		self.headers1 = getHttpHeader1()
		self.headers2 = getHttpHeader2()
		
		# set current agent to header, too
		self.headers.update({b'User-Agent': self.agent})
		self.headers1.update({b'User-Agent': self.agent})
		self.headers2.update({b'User-Agent': self.agent})

		self.path_to_me = getPluginPath()
		path = "%s/skins/%s/screen_main.xml" % (self.path_to_me, config.plugins.skyrecorder.anytime_skin.value)
		with open(path, "r") as f:
			self.skin = f.read()
			f.close()

		#Screen.__init__(self, session)
		MyColorScreen.__init__(self, session)		# support for OriginalMyColor skin

		self.pluginName = config.plugins.skyrecorder.pluginname.value
		self.contentSize = config.plugins.skyrecorder.contentsize.value
		
		self.sky_chlist = buildSkyChannellist()
		
		self.current_group_idx = 0
		
		self.haveInternet = True
		if not checkForInternet():
			self.haveInternet = False
		
		self.popUpIsVisible = False
		self.movieinfoVisible = False
		self.mInfo = SkyTheMovieDB(timeout=10)

		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			actions = ["SkyRecorder_Actions"]
		else:  # dreambox (debian based)
			actions = ["SkyRecorder_Actions_dream"]
		self["mainscreen_actions"]  = ActionMap(actions, {
			"ok"    : self.keyOK,
			"cancel": self.keyCancel,
			"up" : self.keyUp,
			"down" : self.keyDown,
			"right" : self.keyRight,
			"left" : self.keyLeft,
			"nextBouquet" : self.keyPageUp,
			"prevBouquet" : self.keyPageDown,
			"red" : self.skipListe, 
			"green" : self.openWhitelist,
			"yellow" : self.toggleIsNew,
			"blue"	: self.skyarchive, 
			"menu" : self.skysettings,			
			"0" : self.toggleEventIgnored,
			"1" : self.toggleIsNew,
			"2" : self.addToWhitelist,
			"3" : self.toggleIsNew,
			"5" : self.refreshCover,
			"7" : self.previousSort,
			"8" : self.deleteEvent,
			"9" : self.nextSort,
			"nextService" : self.nextGroup,
			"prevService" : self.prevGroup,
			"info" : self.searchMovieInfo,
		}, -1)
		
		#self["popup_actions"] = ActionMap(["OkCancelActions", "ShortcutActions", "EPGSelectActions", "WizardActions", "ColorActions", "NumberActions", "MenuActions", "MoviePlayerActions"],
		self["popup_actions"] = ActionMap(["WizardActions", "OkCancelActions", "ColorActions", "ShortcutActions"],
		{
			"ok": self.setTimerOnOff,
			"cancel": self.keyCancel,
			"up" : self.popUpUp,
			"down" : self.popUpDown,
			"right" : self.popUpPageDown,
			"left" : self.popUpPageUp,
			"nextBouquet" : self.popUpPageUp,
			"prevBouquet" : self.popUpPageDown,
			"yellow": self.toggleShowAllEpisodes,
		}, -1)
		
		self["movieinfo_actions"] = ActionMap(["OkCancelActions", "ShortcutActions", "EPGSelectActions", "WizardActions", "ColorActions", "NumberActions", "MenuActions", "MoviePlayerActions"],
		{
			"ok": self.movieinfoOk,
			"cancel": self.keyCancel,
			"up" : self.movieinfoUp,
			"down" : self.movieinfoDown,
			"right" : self.ignoreKey,
			"left" : self.ignoreKey,
			"red": self.unmatchMovieInfo,
			"green": self.customSearchMovieInfo,
			"nextBouquet" : self.ignoreKey,
			"prevBouquet" : self.ignoreKey
		}, -1)

		self["popup_actions"].setEnabled(False)
		self["movieinfo_actions"].setEnabled(False)

				
		self['title'] = Label(self.pluginName)
		
		self['name'] = Label("lade Sky TV-Guide ...")

		self['handlung'] = ScrollLabel("")

		self['image'] = Pixmap()
		self['image'].hide()

		self['series'] = Pixmap()
		self['series'].hide()

		self['hd'] = Pixmap()
		self['hd'].hide()
		
		self['169'] = Pixmap()
		self['169'].hide()

		self['dolby'] = Pixmap()
		self['dolby'].hide()
		
		self['dualch'] = Pixmap()
		self['dualch'].hide()
		
		self['sub'] = Pixmap()
		self['sub'].hide()
		
		self.ck = {}
		self.keyLocked = True
		
		self.read_system_timers = True
		self.limit_count = 10
		self.limit_offset = 1
		self.listtype = 0
		
		try:
			self.default_before = int(config.recording.margin_before.value)
		except Exception:
			self.default_before = 0
		
		self.red = 0xf23d21
		self.green = 0x389416
		self.blue = 0x0064c7
		self.yellow = 0xbab329
		self.white = 0xffffff
		
		self.streamMenuList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self.streamMenuList.l.setFont(0, gFont('Regular', 30))
		self.streamMenuList.l.setFont(1, gFont('Regular', 28))
		self.streamMenuList.l.setFont(2, gFont('Regular', 26))
		self.streamMenuList.l.setItemHeight(40)
		self['filmliste'] = self.streamMenuList
		
		self.timerSelectList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self.timerSelectList.l.setFont(0, gFont('Regular', 30))
		self.timerSelectList.l.setItemHeight(40)
		self['filmliste_event'] = self.timerSelectList
		self['filmliste_event'].hide()
		self['filmliste_event_bg'] = Pixmap()
		self['filmliste_event_bg'].hide()
		self['filmliste_event_title'] = Label("")
		self['filmliste_event_title'].hide()
		self['filmliste_event_yellow'] = Pixmap()
		self['filmliste_event_yellow'].hide()
		self['filmliste_event_yellow_label'] = Label("Alle / eine Folge")
		self['filmliste_event_yellow_label'].hide()
		if 'my_color_background_2' in self:
			self['my_color_background_2'].hide()  # background of filmliste_event is _2 and movieinfo _3
		self.timerSelectShowAllEpisodes = False

#		self.timerSelectList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
#		self.timerSelectList.l.setFont(0, gFont('Regular', self.contentSize + 6))
#		self.timerSelectList.l.setItemHeight(40)
#		self['filmliste_event'] = self.timerSelectList
#		self['filmliste_event'].hide()
#		self['filmliste_event_bg'] = Pixmap()
#		self['filmliste_event_bg'].hide()

#		self.movieinfoSelectList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
#		self.movieinfoSelectList.l.setFont(0, gFont('Regular', self.contentSize + 6))
#		self.movieinfoSelectList.l.setItemHeight(40)
#		self['movieinfo'] = self.movieinfoSelectList
		self.movieinfoSelectList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self.movieinfoSelectList.l.setFont(0, gFont('Regular', 30))
		self.movieinfoSelectList.l.setItemHeight(40)
		self['movieinfo'] = self.movieinfoSelectList
		self['movieinfo'].hide()
		self['movieinfo_bg'] = Pixmap()
		self['movieinfo_bg'].hide()
		if 'my_color_background_3' in self:
			self['my_color_background_3'].hide()  # background of filmliste_event is _2 and movieinfo _3
		self['movieinfo_red'] = Pixmap()
		self['movieinfo_red'].hide()
		self['movieinfo_green'] = Pixmap()
		self['movieinfo_green'].hide()
		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			self['movieinfo_red_label'] = Label("TMDB entfernen")
		else:  # dreambox (debian based)
			self['movieinfo_red_label'] = Label("Unmatch")
		self['movieinfo_red_label'].hide()
		self['movieinfo_green_label'] = Label("Suche")
		self['movieinfo_green_label'].hide()


		for n in range(0,10):
			star = "star{0}".format(n)
			self[star] = Pixmap()
			self[star].hide()
		
		self.last_index = 0		
		self.retry_count = 0
		self.current_page_number = 1

		self.tempTimer = eTimer()
		self.detailsCoverTimer = eTimer()
		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			self.tempTimer.callback.append(self.getMainEventList)
			self.detailsCoverTimer.callback.append(self.loadDetailsAndCover)
		else:  # dreambox (debian based)
			self.tempTimer_conn = self.tempTimer.timeout.connect(self.getMainEventList)
			self.detailsCoverTimer_conn = self.detailsCoverTimer.timeout.connect(self.loadDetailsAndCover)

		self['filmliste'].onSelectionChanged.append(self.delayedLoadDetailsAndCover)
		self.onLayoutFinish.append(self.delayedGetMainEventList)

		# check if we are ready to go
		self.show_skydb_warning = True
		self.skydb_warning_shown = False
		if config.plugins.skyrecorder.skydb and config.plugins.skyrecorder.skydb.saved_value:
			skydb_path = config.plugins.skyrecorder.skydb.saved_value
			if not re.match('^/usr/.*', skydb_path, re.I) and not (skydb_path == (self.path_to_me + "/skydb.db")):
				self.show_skydb_warning = False

	def delayedGetMainEventList(self, callback=None, infotext="lade ..."):
		if callback:
			self['filmliste'].onSelectionChanged = []  # prevent bluescreen on close
			self.close(self.session, True)
		self['title'].setText(infotext)
		self.tempTimer.start(50, True)


	def delayedLoadDetailsAndCover(self):
		self.detailsCoverTimer.start(400, True)


	def getTimerlist(self):
	
		self.timerList = SkyTimerRec.getTimersList()
		
		#make sure we are in sync with the system timerlist
		#that's why we reset our record status in the database, before updating
		sql.resetEventStatusAdded()
		
		need_commit = False
		for t_record in self.timerList:
			if not t_record['channel']:
				continue
			res = sql.getIdChannel(t_record['channel'],stb=True)
			if not res:
				continue
			id_channel = res
			t_record['id_channel'] = id_channel
			
			starttime = int(t_record['begin'])
			starttime += (config.plugins.skyrecorder.margin_before.value) * 60 # in eventslist we stored the original starttime
			res = sql.getEventsForTimerlist_(t_record['title'], t_record['description'], starttime, id_channel)
			#if not res:
			#	continue
			if not res or len(res) < 1:
				if self.default_before != 0:
					# second check with system margin_before
					starttime += (self.default_before * 60)
					res = sql.getEventsForTimerlist_(t_record['title'], t_record['description'], starttime, id_channel)
					if not res or len(res) < 1:
						continue
			
			#with open("/tmp/sql_debug.txt","a") as debug:
			#	debug.write(t_record['title'] + " " + t_record['description'] + " " + str(starttime) + " " + str(id_channel) + "\n" + str(res) + "\n")
			pk_broadcast = None
			pk_event = None
			for pk_broadcast, id_genre, channel, pk_event in res:
				t_record['id_genre'] = id_genre
				t_record['channel'] = channel
				break
			
			if not pk_event or not pk_broadcast:
				continue
			#if not sql.checkAdded(t_record['title'], t_record['description'], t_record['id_channel'],t_record['id_genre']):
			#if not sql.checkAdded(t_record['title'], t_record['description'], t_record['id_channel'],None):
			#if not sql.checkAdded(t_record['title'], t_record['description'], None,None):
			if not sql.checkAdded_(pk_broadcast, None):
				sql.addAdded(
							t_record['title'],
							t_record['description'],
							t_record['id_channel'],
							t_record['id_genre'],
							t_record['begin'],
							t_record['end'],
							t_record['serviceref'],
							t_record['location'],
							t_record['recordedfile'],
							t_record['eit'],
							int(pk_event),
							int(pk_broadcast))
			# if we still have timers in system timerlist, we want to know about it			
			sql.updateEventStatus(pk_broadcast, starttime, status="True", commit=False)
			need_commit = True
		
		# commit database changes
		if need_commit:
			sql.sqlCommit()


	def backupDatabaseFile(self, renameDB):
		if not renameDB:
			mymsg = "Dann bitte die Datenbank von Hand löschen/umbenennen und den SkyRecorder neustarten!\n"
			mymsg += sql.my_db_path
			self.session.open(MessageBox, mymsg, MessageBox.TYPE_WARNING, timeout=-1)
		else:
			try:
				sql.disconnect()
				os.rename(sql.my_db_path, self.my_db_path_backup)
			except Exception as e:
				self.session.open(MessageBox, "Fehler beim Umbenennen der DB: {}".format(e), MessageBox.TYPE_WARNING, timeout=-1)

	def getMainEventList(self):
		# check if need to show the mean-popup again and again and again...
		if not os.path.exists("/var/lib/dpkg/status"):  # not on dreambox
			if self.show_skydb_warning and not self.skydb_warning_shown:
				from Screens.MessageBox import MessageBox
				message = self.session.open(MessageBox, _("Achtung!\nBitte zuerst den Speicherort der skydb.db Datenbank in den Einstellungen (menu) anpassen.\nDer Pfad sollte zu einem Ort auf einer angeschlossenen Festplatte verweisen."), MessageBox.TYPE_INFO, timeout=-1)
				self.skydb_warning_shown = True

		try:
			sql.cur.execute('SELECT SQLITE_VERSION()')
		except Exception:
			try:
				sql.connect()
			except Exception:
				return

		# check if the database is still compatible or need to be recreated
		try:
			sql.checkIfDatabaseIsStillCompatible()
		except:
			from Screens.MessageBox import MessageBox
			self.my_db_path_backup = sql.my_db_path + datetime.datetime.now().strftime('_%Y%m%d%H%M')
			mymsg = "Die Datenbank ist nicht mehr kompatibel und muss neu erstellt werden!\n"
			#mymsg += "Anschließend muss der SkyRecorder neu gestartet werden!\n"
			mymsg += '\nSoll die Datenbank  "{}"\njetzt (als Backup) in  "{}"\numbenannt werden?'.format(sql.my_db_path, self.my_db_path_backup)
			self.session.openWithCallback(self.backupDatabaseFile, MessageBox, mymsg, MessageBox.TYPE_YESNO, timeout=-1, default=False)
			self['title'].setText("Bitte SkyRecorder beenden und neustarten!")
			return

		# Zeilenhöhe
		if config.plugins.skyrecorder.mainlisttype:
			self.listtype = int(config.plugins.skyrecorder.mainlisttype.value)
			### 0 = one line, 2 = one line with air time ###
			if self.listtype == 0 or self.listtype == 2:
				if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
					self.streamMenuList.l.setItemHeight(37)
				else:  # dreambox (debian based)
					self.streamMenuList.l.setItemHeight(40)
			### 1 = two lines ###
			else:
				self.streamMenuList.l.setItemHeight(75)

		
		if config.plugins.skyrecorder.max_per_page and config.plugins.skyrecorder.max_per_page.value:
			self.limit_count = config.plugins.skyrecorder.max_per_page.value
		else:
			self.limit_count = 10
		
		self.groupnames = []
		self.groupnames.append([0,"Alle"])
		rows = sql.readGroupsShort()
		for t_row in rows:
			row = list(t_row)
			self.groupnames.append(row)
		self.current_group = self.groupnames[self.current_group_idx][1]
		
		if self.read_system_timers:
			self.getTimerlist()
		self.read_system_timers = False
					
		filmliste = []
		
		self.clearFilmInfoScreen()
		
		whitelist = sql.readWhitelist(status="True")
		
		current_timestamp = getCurrentTimestamp()
		
		if config.plugins.skyrecorder.only_active_genres and config.plugins.skyrecorder.only_active_genres.value:
			only_active_genres = True
		else:
			only_active_genres = False

		onlyIsNew = config.plugins.skyrecorder.main_list_show_new_only.value

		if config.plugins.skyrecorder.main_list_order:
			self.sort_type = str(config.plugins.skyrecorder.main_list_order.value)
			self.sort_info = config.plugins.skyrecorder.main_list_order.getText()
		
		try:
			#rows = sql.getEventsMain("ASC", onlyIsNew, self.current_group_idx, current_timestamp, only_active_genres,limit_count=self.limit_count,limit_offset=self.limit_offset,sort_type=self.sort_type,check_endtime=True)
			rows = sql.getEvent2Main("ASC", onlyIsNew, self.current_group_idx, current_timestamp, only_active_genres,limit_count=self.limit_count,limit_offset=self.limit_offset,sort_type=self.sort_type,check_endtime=True)
		except Exception as e:
			print("[skyrecorder] ERROR: {0}".format(e))
			return
		
		resultCount = 0
		if rows:
			resultCount = len(rows)
		canskip = False
		
		# uniq sort_idx needed for correct sorting order
		sort_idx = 0
		if resultCount > 0:
			self.retry_count = 0
			
			need_commit = None
			for t_row in rows:
				row = list(t_row)
								
				# let us see if we have completed recordings. if so, we update status to "Done"
				# id_added, title, description, id_channel, id_genre, begin, end, serviceref, location, recordedfile
				#addedlist_raw = sql.checkAddedReturnEntry(row[1], row[2], row[3], row[5])
				#addedlist_raw = sql.checkAddedReturnEntry(row[1], row[2], row[3], None)
				#addedlist_raw = sql.checkAddedReturnEntry(row[1].lower(), row[2].lower(), row[3], None)
				addedlist_raw = sql.checkAddedReturnEntry_(row[0])  # pk_broadcast
				if addedlist_raw and len(addedlist_raw) != 0:
					for addedlist_raw_row in addedlist_raw:
						addedlist = list(addedlist_raw_row)
						if str(addedlist[9]) == "Hidden":
							#if sql.updateEventListStatus(row[0],e_starttime,status="Hidden",commit=True):
							#if sql.updateEventListStatus(row[0],None,status="Hidden",commit=False):
							if sql.updateEventStatus(row[0], None, status="Hidden", commit=False):
								row[6] = "Hidden"
								need_commit = True
							break
						if int(addedlist[6]) > 0 and int(addedlist[6]) <= current_timestamp:
							if config.plugins.skyrecorder.margin_before and config.plugins.skyrecorder.margin_before.value:
								e_starttime = int(addedlist[5]) + config.plugins.skyrecorder.margin_before.value * 60
							else:
								e_starttime = int(addedlist[5])
							#if sql.updateEventListStatus(row[0],0,"Done"):
							#if sql.updateEventListStatus(row[0],e_starttime,status="Done",commit=False):
							if sql.updateEventStatus(row[0], e_starttime, status="Done", commit=False):
								row[6] = "Done"
								need_commit = True
							break
				addedlist = None
				
				# maybe we have some whitelist entries and want to mark them
				if row[6] != "True":
					for whitelist_event in whitelist:
						escaped1 = str(whitelist_event[2]).lower().replace("[","\[").replace("]","\]").replace("(","\(").replace(")","\)")
						escaped2 = str(row[1]).lower().replace("[","\[").replace("]","\]").replace("(","\(").replace(")","\)")
						if re.match(escaped1 + '.*?', escaped2, re.I):
							row[6] = "Xallowed"
							break
				
				# append sort index
				sort_idx += 1
				row.append(sort_idx)
				filmliste.append(row)				
			
			# commit database changes
			if need_commit:
				sql.sqlCommit()
				need_commit = None
						
			self.streamMenuList.setList(list(map(self.skyAnytimeListEntry, filmliste)))
		else:
			self.limit_offset = 1
			self.current_page_number = 1
			self.retry_count += 1
			if self.retry_count < 2:
				self.delayedGetMainEventList()
				return
			else:
				self.retry_count = 0
		
		if self.last_index < resultCount:
			self['filmliste'].moveToIndex(self.last_index)
		elif resultCount > 1:
			self['filmliste'].moveToIndex(resultCount -1)
		
		try:	
			newnew = ""
			if onlyIsNew:
				newnew = " (Neu)"
			
			self['title'].setText("""Seite: {0}{1}, {2}, {3}""".format(self.current_page_number, newnew, self.current_group, self.sort_info))
		except Exception:
			#sys.exc_clear()
			pass
			
		self.keyLocked = False


	def skyAnytimeListEntry(self, entry):

		if entry[6] == "True":
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_timer.png"
		elif entry[6] == "Done":
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_done.png"
		elif entry[6] == "Hidden":
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_hidden.png"
		elif str(entry[12]) == "1" and entry[6] == "False":
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_is_new.png"
		elif entry[6] == "Xallowed":
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_whitelist.png"
		else:
			iconPath = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_new.png"

		icon = LoadPixmap(iconPath)

		start_time = datetime.datetime.fromtimestamp(int(entry[10]))
		week_day_short = ['So', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa'][int(start_time.strftime('%w'))]

		# next air time e.g. 'Sa 20:15' - 1 line
		nextAirTime = '{}  {}'.format(week_day_short, start_time.strftime('%H:%M'))
		# color of next air time - 1 line
		inOneWeek = datetime.datetime.now().replace(hour=0,minute=0,second=0,microsecond=0) + datetime.timedelta(days=7)
		nextAirTimeColor = self.green
		if start_time >= inOneWeek:
			nextAirTimeColor = self.yellow
		# next air time long - 2 lines
		stime = "ab:  {}  {}".format(week_day_short, getDateTimeFromTimestamp2(entry[10]))

		# show episode-name if this row is in group serie
		#if self.current_group_idx == 2 and entry[4] == "Serie":
		#	category_genre = entry[2]
		#else:
		#	category_genre = entry[4]
		category_genre = entry[2]

		title = entry[1]
		season_no = entry[15]
		episode_no = entry[16]
		episode_title = entry[17]

		if season_no and season_no >= 0:
			if episode_no and episode_no >= 0:
				title += '  ·  S{} E{}'.format(season_no, episode_no)
			else:
				title += '  ·  S{}'.format(season_no, episode_no)

		#if episode_title and episode_title != "":
		#	title += '  ·  {}'.format(episode_title)

		########################################## all (VTi/ATV) ##########################################
		if not os.path.exists("/var/lib/dpkg/status"):
			########## one line (einfach) ##########
			if self.listtype == 0:
				return [entry,
						(eListboxPythonMultiContent.TYPE_TEXT, 2, -2, 42, 30, 0, RT_HALIGN_RIGHT, str(entry[13])),
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 50, 5, 28, 28, icon),
						(eListboxPythonMultiContent.TYPE_TEXT, 85, -2, 400, 40, 0, RT_HALIGN_LEFT, entry[7]),
						(eListboxPythonMultiContent.TYPE_TEXT, 490, -2, 700, 40, 0, RT_HALIGN_LEFT, title),
						(eListboxPythonMultiContent.TYPE_TEXT, 1200, -2, 270, 40, 0, RT_HALIGN_LEFT, category_genre)
						]
			########## two lines (erweitert) ##########
			elif self.listtype == 1:
				return [entry,
						(eListboxPythonMultiContent.TYPE_TEXT, 10, 40, 30, 30, 0, RT_HALIGN_RIGHT, str(entry[13])),
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 10, 5, 28, 28, icon),
						(eListboxPythonMultiContent.TYPE_TEXT, 55, 0, 450, 40, 0, RT_HALIGN_LEFT, entry[7]),
						(eListboxPythonMultiContent.TYPE_TEXT, 55, 40, 400, 35, 0, RT_HALIGN_LEFT, stime, self.green),  # air time long
						(eListboxPythonMultiContent.TYPE_TEXT, 520, 0, 810, 40, 0, RT_HALIGN_LEFT, title),
						(eListboxPythonMultiContent.TYPE_TEXT, 520, 40, 810, 35, 0, RT_HALIGN_LEFT, entry[2], self.red),
						]
			########## one line with air time (einzeilig mit Startzeit) ##########
			else:  #self.listtype == 2
				return [entry,
						(eListboxPythonMultiContent.TYPE_TEXT, 0, 0, 37, 37, 1, RT_HALIGN_RIGHT, str(entry[13])),  # num of events
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 44, 5, 28, 28, icon),  # icon
						(eListboxPythonMultiContent.TYPE_TEXT, 80, 0, 370, 37, 1, RT_HALIGN_LEFT, entry[7]),  # channel
						(eListboxPythonMultiContent.TYPE_TEXT, 465, 0, 615, 37, 0, RT_HALIGN_LEFT, title),
						(eListboxPythonMultiContent.TYPE_TEXT, 1060, 0, 150, 37, 1, RT_HALIGN_RIGHT, nextAirTime, nextAirTimeColor, nextAirTimeColor),  # next air time
						(eListboxPythonMultiContent.TYPE_TEXT, 1230, 0, 270, 37, 1, RT_HALIGN_LEFT, category_genre)  # genre
						]
		########################################## dreambox (debian based) ##########################################
		else:
			########## one line (einfach) ##########
			########## NOT ON DREAMOS: one line with air time (einzeilig mit Startzeit) ##########
			if self.listtype == 0 or self.listtype == 2:
				return [entry,
						(eListboxPythonMultiContent.TYPE_TEXT, 10, 7, 22, 30, 0, RT_HALIGN_RIGHT, str(entry[13])),  # num of events
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 40, 5, 30, 30, icon),  # icon
						(eListboxPythonMultiContent.TYPE_TEXT, 75, 7, 400, 40, 0, RT_HALIGN_LEFT, entry[7]),  # channel
						(eListboxPythonMultiContent.TYPE_TEXT, 480, 7, 600, 40, 0, RT_HALIGN_LEFT, title),
						#(eListboxPythonMultiContent.TYPE_TEXT, 950, 7, 130, 40, 1, RT_HALIGN_RIGHT, nextAirTime, nextAirTimeColor, nextAirTimeColor),  # next air time
						(eListboxPythonMultiContent.TYPE_TEXT, 1150, 7, 270, 40, 0, RT_HALIGN_LEFT, category_genre)  # genre
						]
			########## two lines (erweitert) ##########
			else:
				return [entry,
						(eListboxPythonMultiContent.TYPE_TEXT, 10, 39, 30, 30, 0, RT_HALIGN_RIGHT, str(entry[13])),
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 10, 5, 30, 30, icon),
						(eListboxPythonMultiContent.TYPE_TEXT, 55, 5, 450, 40, 0, RT_HALIGN_LEFT, entry[7]),
						(eListboxPythonMultiContent.TYPE_TEXT, 55, 40, 400, 35, 2, RT_HALIGN_LEFT, stime, self.green),
						(eListboxPythonMultiContent.TYPE_TEXT, 520, 5, 710, 40, 0, RT_HALIGN_LEFT, title),
						(eListboxPythonMultiContent.TYPE_TEXT, 520, 40, 710, 35, 2, RT_HALIGN_LEFT, entry[2], self.red),
						]


	def dataError(self, error):
		self.clearFilmInfoScreen()
		self['name'].setText("Konnte Sky TV-Guide nicht laden.")
		self['handlung'].setText("%s" % error)
		print(error)
		
	def clearFilmInfoScreen(self):
		self.streamMenuList.setList([])
		self['name'].setText("Keine neuen Sendungen gefunden")
		self['image'].hide()
		self['series'].hide()
		self['hd'].hide()
		#self['169'].hide()
		self['dolby'].hide()
		self['dualch'].hide()
		self['sub'].hide()
		self['handlung'].setText("")
		for n in range(0,10):
			star = "star{0}".format(n)
			self[star].hide()


	def loadDetailsAndCover(self):
		try:
			pk_broadcast = self['filmliste'].getCurrent()[0][0]
		except Exception:
			return
		
		for n in range(0,10):
			star = "star{0}".format(n)
			self[star].hide()
		
		self['image'].hide()
		self['series'].hide()
		self['hd'].hide()
		#self['169'].hide()
		self['dolby'].hide()
		self['dualch'].hide()
		self['sub'].hide()
		
		name = self['filmliste'].getCurrent()[0][1]
		if not pk_broadcast and name:
			self['handlung'].setText("")
			self['name'].setText(name)
			return
		
		# get country and year from details
		country = ""
		year = ""
		try:
			rows = sql.getEventCountryYear(pk_broadcast)
			if rows:
				for t_row in rows:
					if t_row[0]:
						country = str(t_row[0])
					if t_row[1]:
						year = str(t_row[1])
					break
		except Exception:
			pass

		#print('### check for MovieInfo ###')

		######  maybe we have some info in our themoviedb-table  ######
		self.m_id_movie = None
		got_movieinfo = False
		sky_title = self['filmliste'].getCurrent()[0][1]
		m_description = None
		rows = sql.getMovieInfo(pk_broadcast=pk_broadcast, m_id_movie=None)
		"""
		0 [pk_themoviedb] INTEGER  PRIMARY KEY NOT NULL,
		1 [fk_broadcast] INTEGER  NOT NULL,
		2 [fk_poster] INTEGER  NULL,
		3 [m_id_movie] INTEGER  NOT NULL,
		4 [m_name] TEXT  NULL,
		5 [m_year] TEXT  NULL,
		6 [m_title_org] TEXT  NULL,
		7 [m_rating] TEXT  NULL,
		8 [m_description] TEXT  NULL,
		9 [m_genre] TEXT  NULL,
		"""
		for t_row in rows:
			got_movieinfo = True

			### activate stars ###
			if float(t_row[7]) > 0:
				max_star = float(t_row[7])
				max_star = int(round(max_star,0))
				for n in range(0,max_star):
					star = "star{0}".format(n)
					self[star].show()

			m_name = str(t_row[4]) + " (" + str(t_row[5]) + ")"
			self['name'].setText(m_name)
			
			m_description = str(t_row[8])

			#print('### check for MovieInfoPoster ###')

			### get movie poster using m_id_movie or pk_broadcast ###
			m_id_movie = int(t_row[3])
			self.m_id_movie = "tmdb_id:{0}".format(m_id_movie)  # need this class-wide for timer tags
			#poster = None
			#if m_id_movie > 0:
			#	poster = sql.getMovieInfoPoster(None, m_id_movie, pos=0)
			#else:
			poster = sql.getMovieInfoPoster(pk_broadcast, None)

			if poster:
				#print('### MovieInfo poster found')
				cover_file = "/tmp/skyrecorder_tempcover.png"
				try:
					with open(cover_file, "wb") as f:
						f.write(poster)
					self.ShowCover(cover_file)
				except Exception as e:
					#return
					print(e)
				finally:
					break
			#else:
			#	print('### MovieInfo poster NOT found')

		#print('### MovieInfo got_movieinfo: ', got_movieinfo )

		if not got_movieinfo:
			image_url = self['filmliste'].getCurrent()[0][8]
			sky_id = self['filmliste'].getCurrent()[0][9]

			#print('### filmliste image_url: ', image_url)

			self['name'].setText(name)

			cover_file = "/tmp/skyrecorder_tempcover.png"
			cover = sql.getBroadcastPoster(pk_broadcast)

			if not cover:
				#print('### no poster mapped in broadcast')
				pk_poster = sql.existsPoster(image_url)
				if pk_poster:
					#print('### link existing poster to broadcast: {}'.format(image_url))
					sql.updateBroadcastPoster(pk_broadcast, pk_poster)
					cover = sql.getBroadcastPoster(pk_broadcast)
				# else:
				# 	print("##### no poster found for url: '{}'".format(image_url))
				# 	go = """SELECT url FROM poster"""
				# 	sql.cur.execute(go)
				# 	urls = sql.cur.fetchall()
				# 	for url in urls:
				# 		print("##### url in db: '{}'".format(url[0]))
			# else:
			# 	print('##### poster was already linked: {}'.format(image_url))

			if cover:
				#print('### sql.getEventCover(pk_broadcast): cover exists')
				try:
					with open(cover_file, "wb") as f:
						f.write(cover)
					self.ShowCover(cover_file)
				except Exception as e:
					print('open/show existing event cover: ', e)
			else:
				#print('### sql.getEventCover(pk_broadcast): no cover')
				try:
					if self.haveInternet:
						#print('### downloadPage: ', image_url)
						downloadPage(image_url.encode('utf-8'), cover_file, headers=self.headers2, agent=self.agent
									).addCallback(self.downloadedImage, cover_file, pk_broadcast, image_url)
				except Exception as e:
					#self.dataError(e)
					print(e)

		#print('### MovieInfo end')

		category = self['filmliste'].getCurrent()[0][2]
		genre = self['filmliste'].getCurrent()[0][4]
		season_no = self['filmliste'].getCurrent()[0][15]
		episode_no = self['filmliste'].getCurrent()[0][16]
		episode_title = self['filmliste'].getCurrent()[0][17]

		rows = sql.getEventDetails(pk_broadcast)
		if rows:
			#print('### found sql.getEventDetails(pk_broadcast)')
			for handlung, series_id, is_hd, is_dolby, is_dualch, is_highlight, is_live, is_new in rows:
				if season_no == -1 and m_description and len(m_description) >= 20:
					# Use the tmdb description only for a movie. For a series the sky tv guide has a better description.
					handlung = m_description
				if not handlung or handlung == "":
					handlung = "Keine Infos gefunden."
				if season_no >= 0:
					if episode_no >= 0:
						last_line = "S{} E{}  ·  {}  ·  {} {}".format(season_no, episode_no, episode_title, country, year)
					else:
						last_line = "S{}  ·  {}  ·  {} {}".format(season_no, episode_title, country, year)
				else:  # a movie
					last_line = "{} {}".format(country, year)
				if category != genre or len(category) > 12:
					last_line += "  ·  {}".format(category)
				if category != genre:
					last_line += "({})".format(genre)
				self['handlung'].setText("{}\n{}".format(handlung, last_line))

				if series_id and series_id > 0:
					self['series'].show()
				if int(is_hd) > 0:
					self['hd'].show()
				#if int(is_169) > 0:
				#	self['169'].show()
				if int(is_dolby) > 0:
					self['dolby'].show()
				if int(is_dualch) > 0:
					self['dualch'].show()
				#if int(is_ut) > 0:
				#	self['sub'].show()
		#else:
		#	print('### not found sql.getEventDetails(pk_broadcast)')

		
	def downloadedImage(self, data, cover_file, pk_broadcast, image_url):
		#print("### downloadedImage: pk_broadcast: {}, image_url: '{}'".format(pk_broadcast, image_url))
		if pk_broadcast:
			#time.sleep(0.123)
			with open(cover_file, "rb") as f:
				cover = f.read() 
				#res = sql.addEventCover(cover_file, cover)
				pk_poster = sql.addPoster(image_url, cover)
				sql.updateBroadcastPoster(pk_broadcast, pk_poster)
		self.ShowCover(cover_file)


	def ShowCover(self, image_path):
		#print('### ShowCover: ', image_path)
		if fileExists(image_path):
			#self.session.open(MessageBox, "show_cover", MessageBox.TYPE_INFO, timeout=3)
			if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
				self['image'].instance.setPixmap(None)
			else:  # dreambox (debian based)
				self['image'].instance.setPixmap(enigma.gPixmapPtr())
			self.scale = AVSwitch().getFramebufferScale()
			self.picload = ePicLoad()
			size = self['image'].instance.size()
			self.picload.setPara((size.width(), size.height(), self.scale[0], self.scale[1], False, 1, "#FF000000"))
			if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
				started = self.picload.startDecode(image_path, 0, 0, False)
			else:  # dreambox (debian based)
				started = self.picload.startDecode(image_path)
				time.sleep(0.3)
			if started == 0:
				ptr = self.picload.getData()
				if ptr != None:
					self['image'].instance.setPixmap(ptr)
					self['image'].show()
					del self.picload
					
	def poster_resize(self, poster_path):
		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			self["poster"].instance.setPixmap(None)
		else:  # dreambox (debian based)
			self["poster"].instance.setPixmap(enigma.gPixmapPtr())
		self["poster"].hide()
		sc = AVSwitch().getFramebufferScale() # Maybe save during init
		self.picload = ePicLoad()
		size = self["poster"].instance.size()
		self.picload.setPara((size.width(), size.height(), sc[0], sc[1], False, 1, "#00000000")) # Background
		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			started = self.picload.startDecode(poster_path, 0, 0, False)
		else:  # dreambox (debian based)
			started = self.picload.startDecode(poster_path)
		if started == 0:
			ptr = self.picload.getData()
			if ptr != None:
				self["poster"].instance.setPixmap(ptr)
				self["poster"].show()
		

	# button actions
	def nextGroup(self):
		self.current_group_idx += 1
		if self.current_group_idx > len(self.groupnames) -1:
			self.current_group_idx = 0
		self.delayedGetMainEventList()
		
	def prevGroup(self):
		self.current_group_idx -= 1
		if self.current_group_idx < 0:
			self.current_group_idx = len(self.groupnames) -1
		self.delayedGetMainEventList()	
	
	def nextSort(self):
		if not config.plugins.skyrecorder.main_list_order or self.keyLocked:
			return
		config.plugins.skyrecorder.main_list_order.handleKey(1)
		config.plugins.skyrecorder.main_list_order.save()
		self.delayedGetMainEventList()
		
	def previousSort(self):
		if not config.plugins.skyrecorder.main_list_order or self.keyLocked:
			return
		config.plugins.skyrecorder.main_list_order.handleKey(0)
		config.plugins.skyrecorder.main_list_order.save()
		self.delayedGetMainEventList()
	
	def toggleEventIgnored(self):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			return
		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		if not pk_broadcast:
			return
		title = self['filmliste'].getCurrent()[0][1]
		desc = self['filmliste'].getCurrent()[0][2]
		id_channel = self['filmliste'].getCurrent()[0][3]
		id_genre = self['filmliste'].getCurrent()[0][5]
		self.last_index = self['filmliste'].getSelectionIndex()
		
		check_state = sql.getEventListStatus_(pk_broadcast, None)
		if check_state and check_state == "Hidden": 
			sql.updateEventStatus(pk_broadcast, None, status="False", commit=True)
			
			#sql.removeFromAdded(title,desc,id_channel,None,hidden=True)
			#sql.removeFromAdded(title.lower(), desc.lower(),None,None,hidden=True)
			sql.removeFromAdded_(pk_broadcast, hidden=True)
			# maybe we should delete this title from our skipword-list instead of disabling?
			if title and len(title) >= 3:
				#sql.delSkip(title)
				sql.setSkipStatus(title, "False")
				
		elif check_state and check_state != "Hidden":
			if check_state == "True":
				addedlist_raw = None
				#addedlist_raw = sql.checkAddedReturnEntry(title.lower(), desc.lower(), id_channel, id_genre)
				#addedlist_raw = sql.checkAddedReturnEntry(title.lower(), desc.lower(), None, None)
				#addedlist_raw = sql.checkAddedReturnEntry(title.lower(), desc.lower(), id_channel, None)
				addedlist_raw = sql.checkAddedReturnEntry_(pk_broadcast)
				if addedlist_raw and len(addedlist_raw) != 0:
					for addedlist_raw_row in addedlist_raw:
						addedlist = list(addedlist_raw_row)
						id_added = addedlist[0]

						timer = SkyTimerRec.getTimerEntry(title, desc, addedlist[5], addedlist[7])
						if timer:
							# delete TMDb cover at location and name of timer record file + .jpg
							if config.plugins.skyrecorder.copy_cover_to_recording.value:
								SkyTimerRec.deleteTMDbCoverOfTimerRecordfile(timer)
							# remove record timer
							SkyTimerRec.removeTimerEntry(timer)
						else:
							return
						# force Hidden and Delete, if we delete a recording timer
						if id_added: 
							sql.resetAddedUpdateHidden(id_added,None)
							res = sql.deleteEventById(pk_broadcast)
			sql.updateEventStatus(pk_broadcast, None, status="Hidden", commit=True)

			# append this title to our skipword-list, but mark it as disabled (-)
			if title and len(title) >= 3:
				sql.addSkip(title, "False")
				
		self.delayedGetMainEventList()


	def deleteEvent(self):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			return
		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		if not pk_broadcast:
			return
		title = self['filmliste'].getCurrent()[0][1]
		desc = self['filmliste'].getCurrent()[0][2]
		id_channel = self['filmliste'].getCurrent()[0][3]
		id_genre = self['filmliste'].getCurrent()[0][5]
		status = self['filmliste'].getCurrent()[0][6]
		self.last_index = self['filmliste'].getSelectionIndex()
		
		check_state = sql.getEventListStatus_(pk_broadcast, False)
		if check_state and (status == "Hidden" or status == "Done"):
			res = sql.deleteEventById(pk_broadcast)
			#if res and not sql.checkAdded(title.lower(), desc.lower(), id_channel, id_genre):
			#if res and not sql.checkAdded(title.lower(), desc.lower(), None, None):
			#if res and not sql.checkAdded(title.lower(), desc.lower(), id_channel, None):
			if res and not sql.checkAdded_(pk_broadcast, None):
				sql.addAdded(title, desc, id_channel, id_genre, 0, 0, '-', '-', 'Hidden', 0, 0, pk_broadcast)
			self.delayedGetMainEventList()
			
			
	def refreshCover(self, pk_broadcast=None, coverurl=None):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			return
		if not pk_broadcast:
			pk_broadcast = self['filmliste'].getCurrent()[0][0]
			if not pk_broadcast:
				return
		self.last_index = self['filmliste'].getSelectionIndex()
		if coverurl:
			# update the cover url for this event in our database
			sql.updateEventCoverUrlByIdEvents(pk_broadcast,coverurl)

		sql.deleteEventCoverByIdEvents(pk_broadcast)
		self.delayedGetMainEventList(infotext="lade Bild neu ...")


	def toggleIsNew(self):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			self.last_index = 0

		config.plugins.skyrecorder.main_list_show_new_only.handleKey(0)
		config.plugins.skyrecorder.main_list_show_new_only.save()

		self.limit_offset = 1
		self.current_page_number = 1
		self.delayedGetMainEventList()
		
	
	def addToWhitelist(self):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			return
		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		if not pk_broadcast:
			return
		title = self['filmliste'].getCurrent()[0][1]
		desc = self['filmliste'].getCurrent()[0][2]
		id_channel = self['filmliste'].getCurrent()[0][3]
		self.last_index = self['filmliste'].getSelectionIndex()
		
		#check_state = sql.addToWhitelist(id_channel,title,desc, status="True")
		check_state = sql.addToWhitelist(id_channel,title,None, status="True")
		self.delayedGetMainEventList()
		
	def openWhitelist(self):
		if self.keyLocked:
			return
		self.session.openWithCallback(self.delayedGetMainEventList, SkyWhitelist)
	

	def skipListe(self):
		self.session.openWithCallback(self.delayedGetMainEventList, SkySkipWordsSelect)


	def skysettings(self):
		self.read_system_timers = True
		self.session.openWithCallback(self.delayedGetMainEventList, SkyRecorderSettings)

	def skyarchive(self):
		self.session.openWithCallback(self.timerCallback, SkyRecorderArchiv)
		return
		new_genres = sql.cur.execute("""SELECT genre.id_genre, genre.genre, genre.status, groups.groupname, groups.id_groups FROM genre
										INNER JOIN genregroup ON genregroup.id_genre == genre.id_genre
										INNER JOIN groups ON groups.id_groups == genregroup.id_groups  
										WHERE genre.id_genre > 87""").fetchall()
		file = '/tmp/new_genres.txt'
		msg = "Neu hinzugefügte Genres, siehe: %s" % file
		with open(file, 'w') as debug_file:
			idx = 0
			for genre in new_genres:
				idx += 1
				debug_file.write("({},'{}','{}'), ".format(genre[0], genre[1], genre[2]))
				if not (idx % 3):
					debug_file.write("\n")
				msg += "\n{},  {},  {}  ->  {}".format(genre[0], genre[1], genre[2], genre[3])
			debug_file.write("\n\n")
			idx = 0
			for genre in new_genres:
				idx += 1
				debug_file.write("({},{}), ".format(genre[0], genre[4]))
				if not (idx % 10):
					debug_file.write("\n")
		self.session.open(MessageBox, msg, MessageBox.TYPE_INFO, timeout=-1)

	def keyPageDown(self):
		if self.keyLocked:
			return
		self['handlung'].pageDown()

	def keyPageUp(self):
		if self.keyLocked:
			return
		self['handlung'].pageUp()

	def keyLeft(self):
		if self.keyLocked:
			return
		if self.limit_offset > self.limit_count:
			self.limit_offset -= self.limit_count
			self.current_page_number -= 1
		else:
			self.limit_offset = 1
			self.current_page_number = 1
		self.delayedGetMainEventList()

	def keyRight(self):
		if self.keyLocked:
			return
		self.limit_offset += self.limit_count
		self.current_page_number += 1
		self.delayedGetMainEventList()
		
		
	def keyUp(self):
		if self.keyLocked:
			return
		self['filmliste'].up()
		self.last_index = self['filmliste'].getSelectionIndex()

	def keyDown(self):
		if self.keyLocked:
			return
		self['filmliste'].down()
		self.last_index = self['filmliste'].getSelectionIndex()
	
	
	def popUpPageDown(self):
		self['filmliste_event'].pageDown()
		
	def popUpPageUp(self):
		self['filmliste_event'].pageUp()
	
	def popUpDown(self):
		self['filmliste_event'].down()
		
	def popUpUp(self):
		self['filmliste_event'].up()
			
	def movieinfoDown(self):
		self['movieinfo'].down()
		
	def movieinfoUp(self):
		self['movieinfo'].up()
		
	def movieinfoOk(self):
		if not self['filmliste'].getCurrent():
			return
		if not self['movieinfo'].getCurrent():
			return
		#movieURL = self['movieinfo'].getCurrent()[0][0]
		#title = self['movieinfo'].getCurrent()[0][2]
		#posterUrl = None
		#posterUrl = self['movieinfo'].getCurrent()[0][5]
		pk_broadcast = self['movieinfo'].getCurrent()[0][0]
		sky_title = self['movieinfo'].getCurrent()[0][1]
		tmdb_result = self['movieinfo'].getCurrent()[0][2]
		self.mInfo.storeMovieInfoAndCoverToDatabase(pk_broadcast=pk_broadcast, sky_title=sky_title, result=tmdb_result, updateAllowed=True)
		self['movieinfo_bg'].hide()
		if 'my_color_background_3' in self:
			self['my_color_background_3'].hide()  # background of filmliste_event is _2 and movieinfo _3
		self['movieinfo'].hide()
		self.movieinfoVisible = False
		self['movieinfo_red_label'].hide()
		self['movieinfo_green_label'].hide()
		self['movieinfo_red'].hide()
		self['movieinfo_green'].hide()
		self["movieinfo_actions"].setEnabled(False)
		self["mainscreen_actions"].setEnabled(True)
		self.timerCallback()
	
	def keyOK(self):
		if self.keyLocked:
			return
		if not self['filmliste'].getCurrent():
			return

		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		if not pk_broadcast:
			return
		self.id_channel = self['filmliste'].getCurrent()[0][3]
		self.id_genre = self['filmliste'].getCurrent()[0][5]
		self.is_new = self['filmliste'].getCurrent()[0][12]
		self.last_index = self['filmliste'].getSelectionIndex()
		check_state = sql.getEventListStatus_(pk_broadcast, False)
		if check_state and check_state == "Hidden":
			return

		self["mainscreen_actions"].setEnabled(False)
		self["popup_actions"].setEnabled(True)
		self['filmliste_event'].show()
		self['filmliste_event_bg'].show()
		self['filmliste_event_title'].show()
		if self['filmliste'].getCurrent()[0][15] >= 0:  # season_no
			self['filmliste_event_yellow'].show()
			self['filmliste_event_yellow_label'].show()
		if 'my_color_background_2' in self:
			self['my_color_background_2'].show()  # background of filmliste_event is _2 and movieinfo _3
		self.popUpIsVisible = True
		self.timerSelectShowAllEpisodes = False  # reset to show one episode not all
		self.getTimerEventList()

	
	def timerCallback(self,status=True):
		self.read_system_timers = status
		self.delayedGetMainEventList()
		
	def ignoreKey(self):
		return
	
	def keyCancel(self):
		if self.popUpIsVisible:
			self['filmliste_event'].hide()
			self['filmliste_event_bg'].hide()
			self['filmliste_event_title'].hide()
			self['filmliste_event_yellow'].hide()
			self['filmliste_event_yellow_label'].hide()
			if 'my_color_background_2' in self:
				self['my_color_background_2'].hide()  # background of filmliste_event is _2 and movieinfo _3
			self.popUpIsVisible = False
			self["popup_actions"].setEnabled(False)
			self["mainscreen_actions"].setEnabled(True)
		elif self.movieinfoVisible:
			self['movieinfo_bg'].hide()
			if 'my_color_background_3' in self:
				self['my_color_background_3'].hide()  # background of filmliste_event is _2 and movieinfo _3
			self['movieinfo'].hide()
			self['movieinfo_red_label'].hide()
			self['movieinfo_green_label'].hide()
			self['movieinfo_red'].hide()
			self['movieinfo_green'].hide()
			self.movieinfoVisible = False
			self["movieinfo_actions"].setEnabled(False)
			self["mainscreen_actions"].setEnabled(True)
		else:
			self['filmliste'].onSelectionChanged = []  # prevent bluescreen on close
			self.close(None, True)


	def skyAnytimerListEntry(self, entry):
		"""
		entry[0]  event.dayname,
		entry[1]  event.datum,
		entry[2]  event.starttime,
		entry[3]  event.endtime,
		entry[4]  channel.channel,
		entry[5]  event.status,
		entry[6]  broadcast.title,
		entry[7]  broadcast.category,
		entry[8]  event.pk_event,
		entry[9]  broadcast.season_no,
		entry[10] broadcast.episode_no
		entry[11] broadcast.new
		"""
		if entry[5] == "True":
			icon = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_timer.png"
		elif entry[5] == "Done":
			icon = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_done.png"
		elif entry[5] == "Hidden":
			icon = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_hidden.png"
		elif entry[11] == 1:
			icon = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_is_new.png"
		else:
			icon = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/images/neu_new.png"
		pix = LoadPixmap(icon)

		weekday = entry[0]
		datum = entry[1][0:6]
		stime = getTimeFromTimestamp(entry[2])
		etime = getTimeFromTimestamp(entry[3])
		times = "{}   -   {}".format(stime, etime)
		channel = entry[4]
		season_no = entry[9]
		episode_no = entry[10]
		series_episode = ""
		if season_no >= 0:
			if episode_no >= 0:
				series_episode = "S{} E{}".format(season_no, episode_no)
			else:
				series_episode = "S{}".format(season_no)

		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			return [entry,
					(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 20, 4, 28, 28, pix),
					(eListboxPythonMultiContent.TYPE_TEXT, 70, 0, 195, 40, 0, RT_HALIGN_LEFT, weekday),
					(eListboxPythonMultiContent.TYPE_TEXT, 265, 0, 115, 40, 0, RT_HALIGN_LEFT, datum),
					(eListboxPythonMultiContent.TYPE_TEXT, 380, 0, 240, 40, 0, RT_HALIGN_LEFT, times),
					(eListboxPythonMultiContent.TYPE_TEXT, 620, 0, 400, 40, 0, RT_HALIGN_LEFT, channel),
					(eListboxPythonMultiContent.TYPE_TEXT, 1045, 0, 110, 40, 0, RT_HALIGN_LEFT, series_episode)
					]
		else:  # dreambox (debian based)
			return [entry,
					(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 20, 3, 30, 30, pix),
					(eListboxPythonMultiContent.TYPE_TEXT, 60, 5, 150, 35, 0, RT_HALIGN_LEFT, weekday),
					(eListboxPythonMultiContent.TYPE_TEXT, 210, 5, 130, 35, 0, RT_HALIGN_LEFT, datum),
					(eListboxPythonMultiContent.TYPE_TEXT, 410, 5, 100, 35, 0, RT_HALIGN_LEFT, stime),
					(eListboxPythonMultiContent.TYPE_TEXT, 510, 5, 100, 35, 0, RT_HALIGN_LEFT, etime),
					(eListboxPythonMultiContent.TYPE_TEXT, 650, 5, 330, 35, 0, RT_HALIGN_LEFT, channel),
					(eListboxPythonMultiContent.TYPE_TEXT, 1045, 5, 110, 35, 0, RT_HALIGN_LEFT, series_episode)
					]
					

	def toggleShowAllEpisodes(self):
		if self['filmliste'].getCurrent() and self['filmliste'].getCurrent()[0][15] == -1:  # season_no
			return  # its not a series, ignore it
		self.timerSelectShowAllEpisodes = not self.timerSelectShowAllEpisodes
		self.getTimerEventList()


	def getTimerEventList(self):
		try:
			sql.cur.execute('SELECT SQLITE_VERSION()')
		except Exception:
			#sys.exc_clear()
			try:
				sql.connect()
			except Exception:
				return

		myList = []
		self.timerSelectList.setList(myList)

		entry = self['filmliste'].getCurrent()
		if not entry:
			return
		pk_broadcast = entry[0][0]
		title = entry[0][1]
		season_no = entry[0][15]
		episode_no = entry[0][16]
		episode_title = entry[0][17]
		if self.timerSelectShowAllEpisodes:
			title += '  ·  ALLE FOLGEN'
		else:
			if season_no and season_no >= 0:
				if episode_no and episode_no >= 0:
					title += '  ·  S{} E{}'.format(season_no, episode_no)
				else:
					title += '  ·  S{}'.format(season_no, episode_no)
			if episode_title and episode_title != "":
				title += '  ·  {}'.format(episode_title)
		self['filmliste_event_title'].setText(title)

		rows = sql.getEvent2Timer(pk_broadcast, "ASC", getCurrentTimestamp(), check_endtime=True, all_episodes=self.timerSelectShowAllEpisodes)
		if rows:
			for t_row in rows:
				myList.append(list(t_row))
			
			self.timerSelectList.setList(list(map(self.skyAnytimerListEntry, myList)))

	def getChannelref(self, channel):	
		for (channelname,channelref) in self.sky_chlist:
			if channelname.lower() == channel.lower():
				return channelref
	
	
	def getEPGevent(self,query,channeref,title,starttime):
		if not query or len(query) != 2:
			return
		epgmatches = []
		epgcache = eEPGCache.getInstance()
		#serviceHandler = eServiceCenter.getInstance()
		allevents = epgcache.lookupEvent(query) or []
		
		for serviceref, eit, name, begin, duration, shortdesc, extdesc in allevents:
			if channeref == serviceref and name.lower() == title.lower() and begin == starttime:
				epgmatches.append( (serviceref, eit, name, begin, duration, shortdesc, extdesc) )
		return epgmatches
				
	def setTimerOnOff(self):
		datum = self['filmliste_event'].getCurrent()[0][1]
		starttime = self['filmliste_event'].getCurrent()[0][2]
		endtime = self['filmliste_event'].getCurrent()[0][3]
		channel = self['filmliste_event'].getCurrent()[0][4]
		status = self['filmliste_event'].getCurrent()[0][5]
		title = self['filmliste_event'].getCurrent()[0][6]
		desc = self['filmliste_event'].getCurrent()[0][7]
		pk_event = self['filmliste_event'].getCurrent()[0][8]
		pk_broadcast_event = self['filmliste_event'].getCurrent()[0][12]
		idx = self['filmliste_event'].getSelectionIndex()
		
		dirname = None
		recordings_base_folder = None
		try:
			if channel == "Sky 3D":
				if config.plugins.skyrecorder.anytimefolder3d.value:
					recordings_base_folder = config.plugins.skyrecorder.anytimefolder3d.value
			else:
				if config.plugins.skyrecorder.anytimefolder.value:
					recordings_base_folder = config.plugins.skyrecorder.anytimefolder.value
		except Exception:
			#sys.exc_clear()
			recordings_base_folder = None
			
		# use settings "margin_before" and "margin_after"
		# for the timers starttime and endtime adjustment
		timer_starttime = starttime - config.plugins.skyrecorder.margin_before.value * 60
		timer_endtime = endtime + config.plugins.skyrecorder.margin_after.value * 60
		
		stb_channel = sql.getChannelFromChannel(channel,stb=True)
		channelref = self.getChannelref(stb_channel)
		
		#print(datum, starttime, endtime, stb_channel, channelref)
		
		# try to delete this recordtimer-entry
		if status == "True":
			timer = SkyTimerRec.getTimerEntry(title, desc, timer_starttime, channelref)
			if not timer and self.default_before != 0:
				default_before_timer_starttime = timer_starttime - (self.default_before * 60)
				timer = SkyTimerRec.getTimerEntry(title, desc, default_before_timer_starttime, channelref)

			if timer:
				# delete TMDb cover at location and name of timer record file + .jpg
				if config.plugins.skyrecorder.copy_cover_to_recording.value:
					SkyTimerRec.deleteTMDbCoverOfTimerRecordfile(timer)

				# remove record timer
				SkyTimerRec.removeTimerEntry(timer)

				# id_added = sql.checkAdded(title.lower(), desc.lower(), self.id_channel, self.id_genre)
				# id_added = sql.checkAdded(title.lower(), desc.lower(), None, None)
				#id_added = sql.checkAdded(title.lower(), desc.lower(), self.id_channel, None)
				id_added = sql.checkAdded_(pk_broadcast_event, None)
				if id_added:
					# reset added
					sql.resetAdded(id_added)

				res = sql.updateEventStatus(pk_broadcast_event, starttime, status="False")

				if config.plugins.skyrecorder.silent_timer_mode.value == False:
					message = self.session.open(MessageBox, _("Timer gelöscht!"), MessageBox.TYPE_INFO, timeout=3)

			self.timerCallback(True)
			self.getTimerEventList()
			self['filmliste_event'].moveToIndex(idx)
			return
		
		# try to add recordtimer-entry
		if channelref != None:
		
			# try to get eventID (eit) from epgCache
			eit = 0
			event_matches = self.getEPGevent(['RITBDSE',(channelref,0, starttime, -1)],channelref,title,starttime)
			if event_matches and len(event_matches) > 0:
				for event_entry in event_matches:
					eit = int(event_entry[1])
					break
			
			justplay = False
			if config.plugins.skyrecorder.timer_mode.value == "1":
				justplay = True
				
			if recordings_base_folder:
				if not config.plugins.skyrecorder.create_dirtree.value:
					dirname = recordings_base_folder
				else:
					# get our groupfoldername
					a_dir = sql.getGenregroupByGenreId(self.id_genre)
					if a_dir:
						group_dir = os.path.join(recordings_base_folder, a_dir + "/")
						if not os.path.exists(group_dir):
							try:
								os.makedirs(group_dir, mode=0o777)
								dirname = group_dir
							except Exception:
								#sys.exc_clear()
								pass
						else:
							dirname = group_dir

			if not config.plugins.skyrecorder.short_record_filenames or not config.plugins.skyrecorder.short_record_filenames.value:
				file = getRecordFilename(title,desc,timer_starttime,stb_channel) # "%s - %s - %s.ts" % (begin_date,channel,title)
			else:
				# for short filenames only need the description field for group "serie" or "series" or "tv shows"
				group = sql.getGenregroupByGenreId(self.id_genre)
				if group.lower() == "serie" or group.lower() == "series" or group.lower() == "tv shows":
					file_base = getRecordFilenameBase(title,desc,timer_starttime,stb_channel)
				else:
					file_base = getRecordFilenameBase(title,"",timer_starttime,stb_channel)

				file = file_base
				ccn = 2
				while 1:
					if not os.path.exists(dirname + file + ".ts"):
						break
					file = file_base + " (" +  str(ccn) +")"
					ccn += 1

			# the suffix ".ts" will be added by the system Timerfunction
			recordfile = Directories.getRecordingFilename(file, dirname)
			#recordfile += ".ts"

			# tags?
			tags = []
			if self.m_id_movie:
				tags.append(self.m_id_movie)
					
			result = SkyTimerRec.addTimer(self.session, channelref, timer_starttime, timer_endtime, title, desc, 0, justplay, 3, dirname, tags, 0, None, eit=eit, recordfile=recordfile)
			if result["result"]:
				sql.updateEventStatus(pk_broadcast_event, starttime, status="True")

				# id_added,title,description,id_channel,id_genre,begin,end,serviceref,location,recordedfile
				#if not sql.checkAdded(title.lower(), desc.lower(), self.id_channel, self.id_genre):
				#if not sql.checkAdded(title.lower(), desc.lower(), None, None):
				#if not sql.checkAdded(title.lower(), desc.lower(), self.id_channel, None):
				if not sql.checkAdded_(pk_broadcast_event, None):
					sql.addAdded(title, desc, self.id_channel, self.id_genre, timer_starttime, timer_endtime, channelref, dirname, file, result["eit"], pk_event, pk_broadcast_event)

				# copy TMDb cover to location and name of timer record file + .jpg
				if config.plugins.skyrecorder.copy_cover_to_recording.value:
					timer = SkyTimerRec.getTimerEntry(title, desc, timer_starttime, channelref)
					if not timer and self.default_before != 0:
						default_before_timer_starttime = timer_starttime - (self.default_before * 60)
						timer = SkyTimerRec.getTimerEntry(title, desc, default_before_timer_starttime, channelref)
					if timer:
						SkyTimerRec.copyTMDbCoverToTimerRecordfile(timer, pk_broadcast=pk_broadcast_event)

				self.timerCallback(True)
				self.getTimerEventList()
				self['filmliste_event'].moveToIndex(idx)
			else:
				message = self.session.open(MessageBox, _("Fehler!\n%s") % (result["message"]), MessageBox.TYPE_INFO, timeout=-1)
				
	
	def customSearchMovieInfo(self):
		pass
		#exist = self['filmliste'].getCurrent()
		#if exist:
		#	title = self['filmliste'].getCurrent()[0][1]
		#else:
		#	title = ""
		#info_title = "Film suchen"
		#self.session.openWithCallback(self.gotMovieSearchName, VirtualKeyBoard, title = info_title, text = title)
	
	
	def gotMovieSearchName(self,title):
		if not title or len(title) < 3:
			return
		self.searchMovieInfo(language="de",title=title)
		
	def unmatchMovieInfo(self):
		if not self['filmliste'].getCurrent():
			return
		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		sql.resetMovieInfoActive(pk_broadcast=pk_broadcast, m_id_movie=None)
		self['movieinfo_bg'].hide()
		if 'my_color_background_3' in self:
			self['my_color_background_3'].hide()  # background of filmliste_event is _2 and movieinfo _3
		self['movieinfo'].hide()
		self['movieinfo_red_label'].hide()
		self['movieinfo_green_label'].hide()
		self['movieinfo_red'].hide()
		self['movieinfo_green'].hide()
		self.movieinfoVisible = False
		self["movieinfo_actions"].setEnabled(False)
		self["mainscreen_actions"].setEnabled(True)
		self.timerCallback()

	def movieinfoListEntry(self, entry):

		#id_entry = entry[0]
		#sky_title = entry[1]
		m_title = entry[2]["m_title"]
		m_date = entry[2]["m_date"]

		if not os.path.exists("/var/lib/dpkg/status"):  # all (VTi/ATV)
			return [entry,
					(eListboxPythonMultiContent.TYPE_TEXT, 20, 0, 980, 40, 0, RT_HALIGN_LEFT, m_title),
					(eListboxPythonMultiContent.TYPE_TEXT, 1010, 0, 120, 40, 0, RT_HALIGN_LEFT, m_date)
					]
		else:  # dreambox (debian based)
			return [entry,
					(eListboxPythonMultiContent.TYPE_TEXT, 20, 5, 680, 30, 0, RT_HALIGN_LEFT, m_title),
					(eListboxPythonMultiContent.TYPE_TEXT, 860, 5, 70, 30, 0, RT_HALIGN_LEFT, m_date)
					]
			
	def searchMovieInfo(self, language="de", title=""):

		if self.keyLocked or not self['filmliste'].getCurrent():
			return

		pk_broadcast = self['filmliste'].getCurrent()[0][0]
		if not pk_broadcast:
			return
		
		#self.last_index = self['filmliste'].getSelectionIndex()
		
		title = self['filmliste'].getCurrent()[0][1]

		is_series = self['filmliste'].getCurrent()[0][14] and self['filmliste'].getCurrent()[0][14] > 0
		# try some fallbacks, if we did not find anything
		tmdb_results = self.mInfo.searchMovieInfos(title, is_series=is_series, language=language)
		# for now, we do not want single matches.
		# This is why we show our resultlist even we have just one match
		
		#self.getMovieInfo(movieURL=res[0]["m_movie_url"],movieTitle=title)
		#if len(res) == 1:
		#	self.getMovieInfo(movieURL=res[0]["m_movie_url"],movieTitle=title)
		#else:
		#	self["mainscreen_actions"].setEnabled(False)
		#	self["movieinfo_actions"].setEnabled(True)
		#	self['movieinfo'].show()
		#	self['movieinfo_bg'].show()
		#	self.movieinfoVisible = True
		#	self.getMovieSearchList(searchlist=res)
			
		self["mainscreen_actions"].setEnabled(False)
		self["movieinfo_actions"].setEnabled(True)
		self['movieinfo'].show()
		self['movieinfo_bg'].show()
		if 'my_color_background_3' in self:
			self['my_color_background_3'].show()  # background of filmliste_event is _2 and movieinfo _3
		self['movieinfo_red_label'].show()
		#self['movieinfo_green_label'].show()
		self['movieinfo_red'].show()
		#self['movieinfo_green'].show()
		self.movieinfoVisible = True
		self.setMovieSearchList(tmdb_results=tmdb_results, pk_broadcast=pk_broadcast, sky_title=title)
		return

	
	def setMovieSearchList(self, tmdb_results=None, pk_broadcast=0, sky_title=""):
		if not tmdb_results:
			tmdb_results = []
		myList = []
		#self.movieinfoSelectList.setList(myList)
		for tmdb_result in tmdb_results:
			myList.append([pk_broadcast, sky_title, tmdb_result])
		self.movieinfoSelectList.setList(list(map(self.movieinfoListEntry, myList)))
		

	def dataErrorPage(self, error):
		self.clearFilmInfoScreen()
		self['name'].setText("Fehler beim Laden.")
		self['handlung'].setText("%s" % error)
		print(error)

