#-*- coding: utf-8 -*-
import re, sqlite3
from Components.config import config
from .SkyMainFunctions import getPluginPath
import os
import re

class sqlcmds:
	def __init__(self):
		self.con = None
		
	def connect(self):
		try:
			self.my_db_path = config.plugins.skyrecorder.skydb.value
		except:
			self.my_db_path = "{0}/skydb.db".format(getPluginPath())

		print(("[SkyRecorder] db path: {0}".format(self.my_db_path)))

		try:
			sizeb = os.path.getsize(self.my_db_path)
		except:
			sizeb = 0
		if not os.path.exists(self.my_db_path) or sizeb == 0:
			from .SkyCreateDatabase import buildSkydb
			res = buildSkydb(self.my_db_path,rebuild=True,backup=False)
		
		self.con = sqlite3.connect(self.my_db_path)
		self.con.text_factory = str # we need utf-8 strings
		self.cur = self.con.cursor()
		self.cur.execute('SELECT SQLITE_VERSION()')
		data = self.cur.fetchone()
		if data:
			return True			
		else:
			return False

	def disconnect(self):
		if self.con:
			self.con.close()


	def sqlCommit(self):
		self.con.commit()


#############################
#### compatibility check ####
#############################

	def checkIfDatabaseIsStillCompatible(self):
		# !actually! if we have the table broadcast, the database is uptodate
		# otherwise it will raise an exception
		self.cur.execute("SELECT pk_broadcast FROM broadcast LIMIT 1")

##########################
#### event statements ####
##########################

	def addBroadcast(self, id_genre, broadcast_id, series_id, title, episode_title, description, category, country,
					 year, season_no, episode_no, image, status, new, hd, dolby, dualch, highlight, live):
		"""
		[pk_broadcast] INTEGER  NOT NULL PRIMARY KEY,
		[fk_genre] INTEGER  NULL,
		[broadcast_id] INTEGER  NULL,
		[series_id] INTEGER  NULL,
		[title] TEXT  NULL,
		[episode_title] TEXT  NULL,
		[description] TEXT  NULL,
		[category] TEXT  NULL,
		[country] TEXT  NULL,
		[year] TEXT  NULL,
		[season_no] INTEGER  NULL,
		[episode_no] INTEGER  NULL,
		[image] TEXT  NULL,
		[status] TEXT  NULL,
		[new] INTEGER NULL,
		[hd] INTEGER NULL,
		[dolby] INTEGER NULL,
		[dualch] INTEGER NULL,
		[highlight] INTEGER NULL,
		[live] INTEGER NULL,
		"""
		pk_broadcast = self.existBroadcast(broadcast_id)
		if pk_broadcast:
			return pk_broadcast

		# add a new broadcast
		title = title.replace("'","''")
		episode_title = episode_title.replace("'","''")
		description = description.replace("'","''")
		image = image.replace("'","''")
		go = """INSERT OR IGNORE INTO 
				broadcast (fk_genre, broadcast_id, series_id, title, episode_title, description, category, country,
						   year, season_no, episode_no, image, status, new, hd, dolby, dualch, highlight, live)
				VALUES ({}, {}, {}, '{}', '{}', '{}', '{}', '{}', '{}', {}, {}, '{}', '{}', {}, {}, {}, {}, {}, {})
				""".format(id_genre, broadcast_id, series_id, title, episode_title, description, category, country,
						   year, season_no, episode_no, image, status, new, hd, dolby, dualch, highlight, live)
		self.cur.execute(go)
		self.con.commit()
		go = "SELECT MAX(pk_broadcast) FROM broadcast"
		self.cur.execute(go)
		pk_broadcasts = self.cur.fetchone()
		if not pk_broadcasts:
			return None
		return int(pk_broadcasts[0])


	def existBroadcast(self, broadcast_id):
		go = """SELECT pk_broadcast FROM broadcast
				WHERE broadcast_id == {0}""".format(broadcast_id)
		self.cur.execute(go)
		pk_broadcasts = self.cur.fetchone()
		if not pk_broadcasts:
			return None
		return int(pk_broadcasts[0])


	def deleteAllBroadcast(self):
		go = """DELETE FROM broadcast"""
		self.cur.execute(go)
		self.con.commit()


	def selectAllBroadcast(self):
		go = """SELECT * FROM broadcast"""
		self.cur.execute(go)
		return self.cur.fetchall()


	def updateBroadcastPoster(self, pk_broadcast, pk_poster):
		go = """UPDATE broadcast SET fk_poster == {} WHERE pk_broadcast == {} """.format(pk_poster, pk_broadcast)
		self.cur.execute(go)
		self.con.commit()


	def addEvent2(self, fk_broadcast, fk_channel, event_id, datum, starttime, endtime, dayname, status="False"):
		"""
		[pk_event] INTEGER  NOT NULL PRIMARY KEY,
		[fk_broadcast] INTEGER  NOT NULL,
		[fk_channel] INTEGER  NOT NULL,
		[event_id] INTEGER  NOT NULL,
		[datum] TEXT  NOT NULL,
		[starttime] INTEGER  NOT NULL,
		[endtime] INTEGER  NOT NULL,
		[dayname] TEXT  NOT NULL,
		[status] TEXT  NULL,
		"""
		# event exist?
		pk_event = self.existEvent2(fk_broadcast, fk_channel, event_id)
		if pk_event:
			return pk_event

		# add a new event
		go = """INSERT OR IGNORE INTO event
				VALUES (NULL, {}, {}, {}, '{}', {}, {}, '{}', '{}')
				""".format(fk_broadcast, fk_channel, event_id, datum, starttime, endtime, dayname, status)
		self.cur.execute(go)
		self.con.commit()
		go = "SELECT MAX(pk_event) FROM event"
		self.cur.execute(go)
		pk_events = self.cur.fetchone()
		if not pk_events:
			return None
		return int(pk_events[0])


	def existEvent2(self, fk_broadcast, fk_channel, event_id):
		go = """SELECT pk_event FROM event
				WHERE fk_broadcast == {}
					AND fk_channel == {}
					AND event_id == {}""".format(fk_broadcast, fk_channel, event_id)
		self.cur.execute(go)
		pk_events = self.cur.fetchone()
		if not pk_events:
			return None
		return int(pk_events[0])


	def deleteAllEvent2(self):
		go = """DELETE FROM event"""
		self.cur.execute(go)
		self.con.commit()


	def selectAllEvent2(self, fk_broadcast):
		go = """SELECT * FROM event WHERE fk_broadcast == {}""".format(fk_broadcast)
		self.cur.execute(go)
		return self.cur.fetchall()


	def existEvent_(self, title, season_no, episode_no, id_channel, starttime, endtime):
		title = title.replace("'", "''")
		go = """SELECT event.pk_event FROM event 
				INNER JOIN broadcast ON event.fk_broadcast = broadcast.pk_broadcast
				WHERE event.fk_channel == {}
					AND event.starttime == {} 
					AND event.endtime == {}
					AND broadcast.season_no == {}
					AND broadcast.episode_no == {}
					AND broadcast.title == '{}' """.format(id_channel, starttime, endtime, season_no, episode_no, title)
		#print('#################### existEventsByTitleChannelStartEnd "{0}"'.format(go))
		self.cur.execute(go)
		return self.cur.fetchall()


	def addEventDetailError(self, title, season_no, episode_no, id_channel, starttime, endtime, event_id):
		title = title.replace("'", "''")
		go = """INSERT OR IGNORE INTO 
				detailerror (fk_channel, event_id, title, season_no, episode_no, starttime, endtime)
				VALUES ({}, {}, '{}', {}, {}, {}, {})
				""".format(id_channel, event_id, title, season_no, episode_no, starttime, endtime)
		self.cur.execute(go)
		self.con.commit()


	def existEventDetailError(self, title, season_no, episode_no, id_channel, starttime, endtime):
		title = title.replace("'", "''")
		go = """SELECT pk_detailerror FROM detailerror 
				WHERE fk_channel == {}
					AND starttime == {} 
					AND endtime == {}
					AND season_no == {}
					AND episode_no == {}
					AND title == '{}' """.format(id_channel, starttime, endtime, season_no, episode_no, title)
		#print('#################### existEventsByTitleChannelStartEnd "{0}"'.format(go))
		self.cur.execute(go)
		return self.cur.fetchall()


	def getEventDetails(self, pk_broadcast):
		go = """SELECT description, series_id, hd, dolby, dualch, highlight, live, new
				FROM broadcast
				WHERE pk_broadcast == {0}""".format(pk_broadcast)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
	
	def getEventCountryYear(self, pk_broadcast):
		go = """SELECT country, year
				FROM broadcast
				WHERE pk_broadcast == {0}""".format(pk_broadcast)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
		
	def resetEventStatus(self):
		go = "UPDATE broadcast SET status == 'False'"
		self.cur.execute(go)
		self.con.commit()
		return True
		

	def resetEventStatusAdded(self):
		go = "UPDATE event SET status == 'False' WHERE status == 'True'"
		self.cur.execute(go)
		self.con.commit()
		return True
	

	def getEventListStatus_(self, pk_broadcast, starttime):
		if not starttime:
			go = "SELECT status FROM event WHERE fk_broadcast=={0}".format(pk_broadcast)
		else:
			go = "SELECT status FROM event WHERE starttime=={0} AND fk_broadcast=={1}".format(starttime, pk_broadcast)
		self.cur.execute(go)
		status = self.cur.fetchone()
		if not status:
			return False
		return str(status[0])


	def updateEventStatus(self, pk_broadcast, starttime, status="False", commit=True):
		if not pk_broadcast:
			return False
		if starttime and int(starttime) > 0:
			go = "UPDATE event SET status = '{2}' WHERE fk_broadcast == {0} AND starttime == {1}".format(pk_broadcast, starttime, status)
		else:
			go = "UPDATE event SET status = '{1}' WHERE fk_broadcast == {0}".format(pk_broadcast, status)
		self.cur.execute(go)
		if commit:
			self.con.commit()
		return True


	def getEvent2Main(self, order="ASC", only_is_new=True, current_group_idx=None, min_date=None,
					  only_active_genres=None, limit_count=10, limit_offset=None, sort_type="begin_asc",
					  check_endtime=False):
		"""
		:return: event[][] <br>
		         event[n][0] = broadcast.pk_broadcast <br>
		         event[n][1] = broadcast.title <br>
		         event[n][2] = broadcast.category <br>
				 event[n][3] = event.fk_channel <br>
				 event[n][4] = genre.genre <br>
				 event[n][5] = genre.id_genre <br>
				 event[n][6] = broadcast.status <br>
				 event[n][7] = channel.channel <br>
				 event[n][8] = broadcast.image <br>
				 event[n][9] = event.event_id <br>
				 event[n][10] = MIN(event.starttime) <br>
				 event[n][11] = event.endtime <br>
				 event[n][12] = broadcast.new <br>
				 event[n][13] = COUNT(event.pk_event) <br>
				 event[n][14] = broadcast.series_id <br>
				 event[n][15] = broadcast.season_no <br>
				 event[n][16] = broadcast.episode_no <br>
				 event[n][17] = broadcast.episode_title <br>
		"""

		# first get skipset
		skipset = self.getSkipSelect()

		self.sort_type = sort_type

		go = """SELECT broadcast.pk_broadcast, broadcast.title, broadcast.category,
					event.fk_channel, genre.genre, genre.id_genre, broadcast.status, channel.channel,
					broadcast.image, event.event_id, MIN(event.starttime), event.endtime,
					broadcast.new, COUNT(event.pk_event) AS anz, broadcast.series_id,
					broadcast.season_no, broadcast.episode_no, broadcast.episode_title 
					FROM event
					INNER JOIN broadcast ON event.fk_broadcast = broadcast.pk_broadcast"""

		if min_date and min_date > 0:
			if check_endtime:
				go += """ AND event.endtime > {0}""".format(min_date)
			else:
				go += """ AND event.starttime > {0}""".format(min_date)

		go += """ INNER JOIN channel ON channel.id_channel = event.fk_channel AND channel.status == 'True'
				  INNER JOIN genre ON genre.id_genre = broadcast.fk_genre"""
		if only_active_genres:
			go += """ AND genre.status == 'True'"""

		if current_group_idx and current_group_idx > 0:
			go += """ INNER JOIN genregroup ON genregroup.id_genre = broadcast.fk_genre
					  INNER JOIN groups ON groups.id_groups = genregroup.id_groups
										   AND groups.id_groups = {0}""".format(current_group_idx)

		if only_is_new:
			go += """ WHERE broadcast.new = 1"""

		if skipset and len(skipset) > 0:
			# go += """ WHERE events.title NOT IN '{0}' """.format(skipset)
			go += " WHERE " + " AND ".join(
				("broadcast.title NOT LIKE '%" + str(n).replace("'", "''").replace("%", "\%") + "%' " for n in skipset))

		if self.sort_type == "channel":
			go += """ GROUP BY event.fk_broadcast ORDER BY channel.channel {0}""".format(order)
		elif self.sort_type == "title":
			go += """ GROUP BY event.fk_broadcast ORDER BY broadcast.title {0}""".format(order)
		elif self.sort_type == "status":
			order = "DESC"
			go += """ GROUP BY event.fk_broadcast ORDER BY broadcast.status {0}""".format(order)
		elif self.sort_type == "begin_desc":
			order = "DESC"
			go += """ GROUP BY event.fk_broadcast ORDER BY MIN(event.starttime) {0}""".format(order)
		elif self.sort_type == "begin_asc":
			go += """ GROUP BY event.fk_broadcast ORDER BY MIN(event.starttime) {0}""".format(order)
		elif self.sort_type == "genre":
			go += """ GROUP BY event.fk_broadcast ORDER BY genre.genre {0}""".format(order)

		if limit_offset and limit_offset > 0:
			if limit_count and limit_count > 0:
				if limit_offset >= 1:  # fix missing first record
					limit_offset -= 1
				go += """ LIMIT {0}, {1}""".format(limit_offset, limit_count)

		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows



	def getEventsMainAutoCheck_(self, order="ASC"):
		"""
		:param order:
		:return: event[][] <br>
			event[n][0] = broadcast.pk_broadcast <br>
			event[n][1] = broadcast.title <br>
			event[n][2] = broadcast.category <br>
			event[n][3] = event.fk_channel <br>
			event[n][4] = genre.genre <br>
			event[n][5] = genre.id_genre <br>
			event[n][6] = broadcast.status <br>
			event[n][7] = channel.channel <br>
			event[n][8] = broadcast.image <br>
			event[n][9] = event.event_id <br>
			event[n][10] = event.starttime <br>
			event[n][11] = event.endtime <br>
			event[n][12] = broadcast.new <br>
			event[n][13] = genre.status <br>
			event[n][14] = themoviedb.pk_themoviedb or 0 <br>
			event[n][15] = broadcast.year <br>
			event[n][16] = broadcast.series_id
		"""
		go = """SELECT broadcast.pk_broadcast, broadcast.title, broadcast.category,
						event.fk_channel, genre.genre, genre.id_genre, broadcast.status, channel.channel,
						broadcast.image, event.event_id, event.starttime, event.endtime, broadcast.new,
						genre.status, IFNULL(themoviedb.pk_themoviedb,0) AS tmdb, broadcast.year,
						broadcast.series_id
						FROM event
						INNER JOIN broadcast ON event.fk_broadcast = broadcast.pk_broadcast 
						INNER JOIN channel ON channel.id_channel = event.fk_channel AND channel.status == 'True'
						INNER JOIN genre ON genre.id_genre = broadcast.fk_genre
						LEFT JOIN themoviedb ON themoviedb.fk_broadcast = broadcast.pk_broadcast
						WHERE broadcast.status != 'Hidden'
					GROUP BY event.fk_broadcast
					ORDER BY broadcast.new DESC, MIN(event.starttime) {0}""".format(order)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows


	def getEvent2Timer(self, pk_broadcast, order="ASC", min_date=None, check_endtime=False, all_episodes=False):
		series_id = None
		if all_episodes:
			series_id = self.cur.execute("SELECT series_id FROM broadcast WHERE pk_broadcast == ?", [pk_broadcast]).fetchone()[0]

		go = """SELECT event.dayname, event.datum, event.starttime, event.endtime, channel.channel, event.status, 
						broadcast.title, broadcast.category, event.pk_event, broadcast.season_no, broadcast.episode_no, 
						broadcast.new, broadcast.pk_broadcast
				FROM event
				INNER JOIN broadcast ON broadcast.pk_broadcast = event.fk_broadcast
				INNER JOIN channel ON event.fk_channel = channel.id_channel """
		if series_id and series_id != 0:
			# all events of all episodes of this series
			go += """ WHERE broadcast.series_id == {} """.format(series_id)
		else:
			# all events from this movie or from this episode
			go += """ WHERE event.fk_broadcast == {} """.format(pk_broadcast)
		if min_date and min_date > 0:
			if check_endtime:
				go += """ AND event.endtime > {} """.format(min_date)
			else:
				go += """ AND event.starttime > {} """.format(min_date)
		if series_id and series_id != 0:
			go += """ ORDER BY broadcast.season_no ASC, broadcast.episode_no ASC, event.starttime {} """.format(order)
		else:
			go += """ ORDER BY event.starttime {} """.format(order)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows


	def getEventsForTimerlist_(self, title, description, starttime, id_channel):
		title = title.replace("'","''")
		description = description.replace("'","''")
		go = """SELECT event.fk_broadcast, broadcast.fk_genre, channel.channel, event.pk_event
				FROM event
				INNER JOIN broadcast ON broadcast.pk_broadcast = event.fk_broadcast
				INNER JOIN channel ON channel.id_channel = event.fk_channel
				WHERE broadcast.title LIKE '{0}'
					AND broadcast.category LIKE '{1}'
					AND event.starttime == {2}
					AND event.fk_channel == {3}
				GROUP BY event.fk_broadcast """.format(title, description, starttime, id_channel)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows


	def updateEventCoverUrlByIdEvents(self, pk_broadcast=None, url=None):
		if not pk_broadcast or not url:
			return
		go = "UPDATE broadcast SET image = '{0}' WHERE pk_broadcast=={1}".format(url, pk_broadcast)
		self.cur.execute(go)
		self.con.commit()


	def getBroadcastPoster(self, pk_broadcast):
		go = """SELECT poster.data FROM poster
				INNER JOIN broadcast ON broadcast.fk_poster == poster.pk_poster 
				WHERE broadcast.pk_broadcast == {}
				LIMIT 1 """.format(pk_broadcast)
		self.cur.execute(go)
		data = self.cur.fetchone()
		if data:
			return data[0]  #str(data[0])
		else:
			return None
	
	
	def deleteEvents(self, mindate):
		go = """SELECT event.fk_broadcast
				FROM event
				WHERE event.starttime <= {0}""".format(mindate)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		n = 0
		if rows:
			for pk_broadcast in rows:
				n += 1
				go = """DELETE FROM event
					WHERE event.fk_broadcast == {0}
					AND event.starttime <= {1}""".format(pk_broadcast[0], mindate)
				self.cur.execute(go)
				self.con.commit()
				go = """SELECT COUNT(event.fk_broadcast) FROM event
					WHERE event.fk_broadcast == {0}""".format(pk_broadcast[0])
				self.cur.execute(go)
				count = self.cur.fetchone()
				if count and int(count[0]) < 1:
					#go = """DELETE FROM eventcover
					#	WHERE eventcover.id_events == {0}""".format(id_events[0])
					#self.cur.execute(go)
					#self.con.commit()
					try:
						go = """DELETE FROM broadcast
							WHERE broadcast.pk_broadcast == {0}""".format(pk_broadcast[0])
						self.cur.execute(go)
						self.con.commit()
					except:
						return -1
		return n


	def deleteDetailErrors(self, olderThanYesterday):
		go = """SELECT COUNT(*) FROM detailerror"""
		self.cur.execute(go)
		before = int(self.cur.fetchone()[0])

		go = """DELETE FROM detailerror
				WHERE starttime <= {} """.format(olderThanYesterday)
		self.cur.execute(go)
		self.con.commit()

		go = """SELECT COUNT(*) FROM detailerror"""
		self.cur.execute(go)
		after = int(self.cur.fetchone()[0])

		return before - after

		
	def deleteEventById(self, pk_broadcast):
		if not pk_broadcast or pk_broadcast < 1:
			return False

		#go = """DELETE FROM eventcover WHERE id_events == {0}""".format(pk_broadcast)
		#self.cur.execute(go)
		#self.con.commit()
		
		go = """DELETE FROM event WHERE fk_broadcast == {0}""".format(pk_broadcast)
		self.cur.execute(go)
		self.con.commit()
		
		return True


	def existBroadcastIsNew(self, title, episode_title, season_no, episode_no, country, year):
		title = title.replace("'", "''")
		episode_title = episode_title.replace("'", "''")
		go = """SELECT pk_broadcast
				FROM broadcast
				WHERE title == '{}'
					AND episode_title == '{}'
					AND season_no == {}
					AND episode_no == {}
					AND country == '{}'
					AND year == '{}'
					AND new == 1
					LIMIT 1 """.format(title, episode_title, season_no, episode_no, country, year)
		self.cur.execute(go)
		pk_broadcasts = self.cur.fetchone()
		if not pk_broadcasts:
			return None
		return int(pk_broadcasts[0])


##########################
#### genre statements ####
##########################
		
	def changeGenre(self, genrename):
		print(genrename)
		go = "SELECT Status FROM genre WHERE genre == '{0}'".format(genrename)
		self.cur.execute(go)
		data = self.cur.fetchone()
		print(data)
		if data[0] == "True":
			print("is True set False")
			go = "UPDATE genre SET status == 'False' WHERE genre LIKE '{0}'".format(genrename)
			self.cur.execute(go)
			self.con.commit()
		else:
			print("is False set True")
			go = "UPDATE genre SET status == 'True' WHERE genre LIKE '{0}'".format(genrename)
			self.cur.execute(go)
			self.con.commit()

	def checkGenre(self,genre):
		if self.existGenre(genre):
			go = "SELECT status FROM genre WHERE genre LIKE '{0}'".format(genre)
			self.cur.execute(go)
			status = self.cur.fetchone()
			self.con.commit()
			if status[0] == "True":
				return True
			else:
				return False
		else:
			return False

	def existGenre(self,genre):
		go = "SELECT COUNT(*) FROM genre WHERE genre LIKE '{0}'".format(genre)
		self.cur.execute(go)
		count = self.cur.fetchone()
		if count and int(count[0]) > 0:
			return True
		else:
			return False
			
	def getGenreFromId(self,id):
		go = "SELECT genre FROM genre WHERE id_genre == '{0}'".format(id)
		self.cur.execute(go)
		genre_name = self.cur.fetchone()
		if genre_name:
			return genre_name[0]
		else:
			return False		
			
	def getIdGenre(self,genre):
		if not genre:
			return False
		genre = genre.replace("'","''")
		go = "SELECT id_genre FROM genre WHERE genre LIKE '{0}'".format(genre)
		self.cur.execute(go)
		id_genre = self.cur.fetchone()
		if id_genre:
			return id_genre[0]
		else:
			return False
			
	def readGenre(self):
		go = "SELECT genre, status FROM genre"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows

	def getActiveGenre(self):
		go = "SELECT id_genre FROM genre WHERE status == 'True'"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		templist = []
		for row in rows:
			templist.append(row[0])
		return tuple(templist)

	def readGenreJoinGenregroup(self):
		go = """SELECT genre.genre, genre.status, genregroup.id_genregroup, genregroup.id_genre, genregroup.id_groups
				FROM genre
				INNER JOIN genregroup ON genregroup.id_genre = genre.id_genre
				"""
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows

	def addGenre(self, genre_name, id_groups):
		next_id_genre = self.cur.execute("SELECT MAX(id_genre) FROM genre").fetchone()[0] + 1
		self.cur.execute("INSERT INTO genre (id_genre, genre, status) values (?,?,?)", [next_id_genre, genre_name, 'False'])
		self.cur.execute("INSERT INTO genregroup (id_genre, id_groups) values (?,?)", [next_id_genre, id_groups])
		self.con.commit()
		return next_id_genre

###########################
#### groups statements ####
###########################

	def getGroupnames(self):
		go = "SELECT groupname FROM groups"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows

	def readGroupsShort(self):
		go = "SELECT id_groups,groupname FROM groups"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
	def readJoindGroupsShort(self, id_genregroup):
		go = """SELECT groups.id_groups, groups.groupname
				FROM groups
				INNER JOIN genregroup ON genregroup.id_genregroup = {0}
				INNER JOIN genre ON genre.id_genre = genregroup.id_genre
				""".format(id_genregroup)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
	def readGenregroup(self):
		go = "SELECT id_genregroup,id_groups,id_genre FROM genregroup"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows

	def updateGenregroup(self,id_genregroup, id_groups, do_commit=True):
		go = """UPDATE genregroup
				SET id_groups == {1}
				WHERE id_genregroup == {0}""".format(id_genregroup, id_groups)
		self.cur.execute(go)
		if do_commit:
			self.con.commit()


	def getGenregroupByGenre(self,genre):
		if not genre:
			return False
		if genre == "" or genre == " ":
			genre = "-"
		else:
			genre = genre.replace("'","''")
		go = """SELECT groups.groupname
				FROM groups
				INNER JOIN genregroup ON genregroup.id_groups = groups.id_groups
				INNER JOIN genre ON genregroup.id_genre = genre.id_genre
				AND genre.genre == '{0}'
				""".format(genre)
		self.cur.execute(go)
		row = self.cur.fetchone()
		if row:
			return row[0]
		else:
			return False


	def getGenregroupByGenreId(self,id_genre):
		if not id_genre:
			return False
		go = """SELECT groups.groupname
				FROM groups
				INNER JOIN genregroup ON genregroup.id_groups = groups.id_groups
				INNER JOIN genre ON genregroup.id_genre = genre.id_genre
				AND genre.id_genre == {0}
				""".format(id_genre)
		self.cur.execute(go)
		row = self.cur.fetchone()
		if row:
			return row[0]
		else:
			return False

############################		
#### channel statements ####
############################

	def changeChannel(self,channel=""):
		channel = channel.replace("'","''")
		channel = channel.replace("%","\%")
		go = "SELECT status FROM channel WHERE channel == '{0}'".format(channel)
		self.cur.execute(go)
		data = self.cur.fetchone()
		print(data)
		if data[0] == "True":
			print("is True set False")
			go = "UPDATE channel SET status == 'False' WHERE channel LIKE '{0}'".format(channel)
			self.cur.execute(go)
			self.con.commit()
		else:
			print("is False set True")
			go = "UPDATE channel SET status == 'True' WHERE channel LIKE '{0}'".format(channel)
			self.cur.execute(go)
			self.con.commit()

	def checkChannel(self,channel=""):
		if self.existChannel(channel):
			channel = channel.replace("'","''")
			channel = channel.replace("%","\%")
			go = "SELECT status FROM channel WHERE channel LIKE '{0}'".format(channel)
			self.cur.execute(go)
			status = self.cur.fetchone()
			self.con.commit()
			if status[0] == "True":
				return True
			else:
				return False
		else:
			return False
			
	def existChannel(self,channel=""):
		channel = channel.replace("'","''")
		channel = channel.replace("%","\%")
		go = "SELECT COUNT(*) FROM channel WHERE channel LIKE '{0}'".format(channel)
		self.cur.execute(go)
		count = self.cur.fetchone()
		if count and int(count[0]) > 0:
			return True
		else:
			return False
			
			
	def existChannelBySkyId(self, channel_id_sky):
		if not channel_id_sky or channel_id_sky < 1:
			return False
		go = "SELECT COUNT(*) FROM channel WHERE channel_id_sky == {0}".format(channel_id_sky)
		self.cur.execute(go)
		count = self.cur.fetchone()
		if count and int(count[0]) > 0:
			return True
		else:
			return False


	def updateChannelName(self, channel_id_sky, channel_name):
		if not channel_id_sky or channel_id_sky < 1 or not channel_name or channel_name == "":
			return None
		channel_name = channel_name.replace("'", "''")
		channel_name = channel_name.replace("%", "\%")

		go = "SELECT id_channel, channel FROM channel WHERE channel_id_sky == {0} AND NOT channel LIKE '{1}'".format(channel_id_sky, channel_name)
		self.cur.execute(go)
		row = self.cur.fetchone()
		if row:
			id_channel = row[0]
			old_channel_name = row[1]
			go = "UPDATE channel SET channel == '{0}' WHERE id_channel == {1}".format(channel_name, id_channel)
			self.cur.execute(go)
			self.con.commit()
			return old_channel_name
		else:
			return None

			
	def getIdChannel(self, channel="", stb=False):
		channel = channel.replace("'","''")
		channel = channel.replace("%","\%")
		if stb:
			go = "SELECT id_channel FROM channel WHERE channel_stb LIKE '{0}'".format(channel)  # never used
		else:
			go = "SELECT id_channel FROM channel WHERE channel LIKE '{0}'".format(channel)
		self.cur.execute(go)
		id_channel = self.cur.fetchone()
		if id_channel:
			return id_channel[0]
		else:
			return False
			
	def getChannelFromChannel(self,channel="",stb=False):
		channel = channel.replace("'","''")
		channel = channel.replace("%","\%")
		if stb:
			go = "SELECT channel_stb FROM channel WHERE channel LIKE '{0}'".format(channel)
		else:
			go = "SELECT channel FROM channel WHERE channel_stb LIKE '{0}'".format(channel)
		self.cur.execute(go)
		id_channel = self.cur.fetchone()
		if id_channel:
			return id_channel[0]
		else:
			return False

	def readChannel(self):
		go = "SELECT channel,status FROM channel"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
	
	def getActiveChannel(self):
		go = "SELECT id_channel FROM channel WHERE status == 'True'"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		templist = []
		for row in rows:
			templist.append(row[0])
		return tuple(templist)

	def getChannelIdSky(self, active=True):
		if active:
			go = "SELECT channel_id_sky FROM channel WHERE status == 'True'"
		else:
			go = "SELECT channel_id_sky FROM channel"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		templist = []
		for row in rows:
			templist.append(row[0])
		return templist
	
	def readChannelAll(self):
		go = "SELECT * FROM channel"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
	
	def updateChannelnameSTB(self,id_channel, channel_stb):
		if not channel_stb:
			return False
		channel_stb = channel_stb.replace("'","''")
		channel_stb = channel_stb.replace("%","\%")
		go = "UPDATE channel SET channel_stb == '{0}' WHERE id_channel == {1}".format(channel_stb,id_channel)
		self.cur.execute(go)
		self.con.commit()
		
	def addNewChannel(self,channel, channel_stb, status, position, channel_id_sky):
		if not channel or not channel_stb or not channel_id_sky:
			return False
		channel = channel.replace("'","''")
		channel_stb = channel_stb.replace("'","''")
		channel = channel.replace("%","\%")
		channel_stb = channel_stb.replace("%","\%")
		if self.existChannel(channel):
			go = """UPDATE channel
				SET
				channel_id_sky == {0},
				channel == '{1}'
				WHERE channel LIKE '{1}'""".format(channel_id_sky,channel)
		else:
			go = """INSERT OR IGNORE INTO channel
				(channel,channel_stb,status,position,channel_id_sky)
				VALUES ('{0}','{1}','{2}',{3},{4})
				""".format(channel,channel_stb,status,position,channel_id_sky)
		self.cur.execute(go)
		self.con.commit()
		return self.existChannelBySkyId(channel_id_sky)
	
	def deleteChannel(self,id_channel):
		if not id_channel:
			return
		go = "DELETE FROM channel WHERE id_channel == {0}".format(id_channel)
		self.cur.execute(go)
		self.con.commit()
		
	def truncateTableChannel(self):
		go = "DELETE FROM channel"
		self.cur.execute(go)
		self.con.commit()
	
################################	
#### timer added statements ####
################################
	
	
	# id_added,title,description,id_channel,id_genre,begin,end,serviceref,location,recordedfile
		
	def addAdded(self, title, description, id_channel, id_genre, begin, end, serviceref, location, recordedfile, eit, pk_event, pk_broadcast):
		#if not self.checkAdded(title,description,id_channel,id_genre):
		#if not self.checkAdded(title,description,id_channel,None):
		if not self.checkAdded_(pk_broadcast, None):
			title = title.replace("'","''")
			description = description.replace("'","''")
			recordedfile = recordedfile.replace("'","''")
			go = """INSERT OR IGNORE INTO added
					(title, category, id_channel, id_genre, begin, end, serviceref, location, recordedfile, eit, fk_event, fk_broadcast)
					VALUES
					('{0}', '{1}', {2}, {3}, {4}, {5}, '{6}', '{7}', '{8}', {9}, {10}, {11})
					""".format(title, description, id_channel, id_genre, begin, end, serviceref, location, recordedfile, eit, pk_event, pk_broadcast)
			self.cur.execute(go)
			#self.cur.fetchone()
			self.con.commit()
			return True
		else:
			return False


	def checkAdded_(self, pk_broadcast, hidden=None):
		go = """SELECT pk_added FROM added
				WHERE fk_broadcast == {}""".format(pk_broadcast)
		if hidden and hidden == True:
			go += """ AND begin == 0"""
		self.cur.execute(go)
		pk_added = self.cur.fetchone()
		if pk_added:
			return int(pk_added[0])
		else:
			return False


	def checkAdded(self,title,description,id_channel,id_genre,hidden=None):
		title = title.replace("'","''")
		title = title.replace("%","\%")
		description = description.replace("'","''")
		description = description.replace("%","\%")
		go = """SELECT id_added FROM added
				WHERE title LIKE '{0}' AND
				description LIKE '{1}' """.format(title,description)
		#nur bei Sky 3D zusätzlich den Channel prüfen und eventuell doppelt aufnehmen
		if id_channel and id_channel == self.getIdChannel("Sky 3D",stb=False):
			go += """ AND id_channel = {0}""".format(id_channel)
		if id_genre:
			go += """ AND id_genre = {0}""".format(id_genre)
		if hidden and hidden == True:
			go += """ AND begin == 0"""
		self.cur.execute(go)
		id_added = self.cur.fetchone()
		if id_added:
			return int(id_added[0])
		else:
			return False


	def checkAddedReturnEntry_(self, pk_broadcast):
		go = """SELECT * FROM added
				WHERE fk_broadcast == {}""".format(pk_broadcast)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		if rows:
			return rows
		else:
			return False


	def checkAddedReturnEntry(self,title,description,id_channel,id_genre):
		title = title.replace("'","''")
		title = title.replace("%","\%")
		description = description.replace("'","''")
		description = description.replace("%","\%")
		go = """SELECT * FROM added
				WHERE title LIKE '{0}' AND
				description LIKE '{1}' """.format(title,description)
		#nur bei Sky 3D zusätzlich den Channel prüfen und eventuell doppelt aufnehmen
		if id_channel and id_channel == self.getIdChannel("Sky 3D",stb=False):
			go += """ AND id_channel = {0}""".format(id_channel)
		if id_genre:
			go += """ AND id_genre = {0}""".format(id_genre)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		if rows:
			return rows
		else:
			return False

	
	def resetAddedUpdateHidden(self, pk_added=None, pk_event=None):
		go = "UPDATE added SET begin = 0, end = 0, recordedfile = 'Hidden'"
		if pk_added and pk_added > 0:
			go += " WHERE pk_added == {0}".format(pk_added)
		elif pk_event and pk_event > 0:
			go += " WHERE fk_event == {0}".format(pk_event)
		self.cur.execute(go)
		self.con.commit()
		
	def resetAdded(self, pk_added=None):
		go = "DELETE FROM added"
		if pk_added and pk_added > 0:
			go += " WHERE pk_added == {0}".format(pk_added)
		self.cur.execute(go)
		self.con.commit()		

	def removeFromAdded_(self, pk_broadcast, hidden=False):
		go = """DELETE FROM added
				WHERE fk_broadcast == {}""".format(pk_broadcast)
		if hidden and hidden == True:
			go += """ AND begin == 0"""
		self.cur.execute(go)
		self.con.commit()

	def removeFromAdded(self,title,description,id_channel,id_genre,hidden=False):
		title = title.replace("%","\%")
		title = title.replace("'","''")
		description = description.replace("'","''")
		description = description.replace("%","\%")
		go = """DELETE FROM added WHERE
					title LIKE '{0}' AND 
					description LIKE '{1}'""".format(title,description)
		#nur bei Sky 3D zusätzlich den Channel prüfen und eventuell doppelt aufnehmen
		if id_channel and id_channel == self.getIdChannel("Sky 3D",stb=False):
			go += """ AND id_channel = {0}""".format(id_channel)
		if id_genre and id_genre > 0:
			go += """ AND id_genre = {0}""".format(id_genre)
		
		if hidden and hidden == True:
			go += """ AND begin == 0"""
		self.cur.execute(go)
		self.con.commit()
			
	def readAdded(self):
		go = "SELECT recordedfile FROM added"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		

	def readAddedEdit(self):
		go = """SELECT pk_added, title, category, id_channel, id_genre, recordedfile, begin, fk_event
				FROM added
				ORDER BY begin ASC"""
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows

	def readAddedAll(self):
		go = "SELECT * FROM added"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows


	def readAddedByGroupname(self,groupname=None):
		if not groupname:
			return
		groupname = groupname.replace("'","''")
		go = """SELECT * FROM added
				INNER JOIN genre ON genre.id_genre == added.id_genre
				INNER JOIN genregroup ON genregroup.id_genre == genre.id_genre
				INNER JOIN groups ON groups.id_groups == genregroup.id_groups"""
		if groupname.lower() == "all" or groupname.lower() == "a-z":
			go += """ ORDER BY added.title ASC"""
		else: 
			go += """ WHERE groups.groupname == '{0}' ORDER BY added.title ASC""".format(groupname)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows


#############################	
#### skipword statements ####
#############################

	def changeSkip(self,skipword):
		if not skipword:
			return
		skipword = skipword.replace("'","''")
		skipword = skipword.replace("%","\%")
		go = "SELECT status FROM skip WHERE skipword == '{0}'".format(skipword) 
		self.cur.execute(go)
		data = self.cur.fetchone()
		print(data)
		if data[0] == "True":
			print("is True set False")
			go = "UPDATE skip SET status == 'False' WHERE skipword LIKE '{0}'".format(skipword)
			self.cur.execute(go)
			self.con.commit()
		else:
			print("is False set True")
			go = "UPDATE skip SET status == 'True' WHERE skipword LIKE '{0}'".format(skipword)
			self.cur.execute(go)
			self.con.commit()
			
	def checkSkip(self,skipword):
		if not skipword:
			return
		skipword = skipword.replace("'","''")
		skipword = skipword.replace("%","\%")
		go = "SELECT status FROM skip WHERE skipword LIKE '{0}'".format(skipword)
		self.cur.execute(go)
		status = self.cur.fetchone()
		self.con.commit()
		if status:
			if status[0] == "True":
				return True
			else:
				return False
		else:
			return False

	def readSkip(self):
		go = "SELECT skipword,status FROM skip WHERE status == 'True'"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
			
	def readSkipSelect(self):
		go = "SELECT skipword,status FROM skip"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
	
	def getSkipSelect(self):
		# TODO: needs to be pure and clean sql-query
	
		#go = """SELECT skip.skipword
		#		FROM skip
		#		INNER JOIN whitelist ON whitelist.title != skip.skipword
		#		WHERE skip.status == 'True'
		#		GROUP BY skip.skipword COLLATE NOCASE"""
		whitelist = self.getActiveWhitelistTitle()
		if not whitelist or len(whitelist)< 1:
			whitelist = []
		#go = """SELECT skip.skipword
		#		FROM skip
		#		INNER JOIN whitelist ON whitelist.title NOT LIKE '%' + skip.skipword + '%' WHERE skip.status == 'True'
		#		GROUP BY skip.skipword"""
		
		go = """SELECT skipword FROM skip WHERE status == 'True' GROUP BY skipword"""
		self.cur.execute(go)
		rows = self.cur.fetchall()
		templist = []
		for row in rows:
			canskip = None
			skipword = str(row[0])
			for white in whitelist:
				if re.match('.*?' + skipword.replace("[","\[").replace("]","\]") + '.*?', str(white[0]), re.I):
					canskip = True
					break
			if not canskip:
				templist.append(skipword)
		return tuple(templist)
	
	def addSkip(self,skipword, status="True"):
		if not skipword:
			return
		if not self.existSkip(skipword):
			skipword = skipword.replace("'","''")
			go="INSERT OR IGNORE INTO skip (skipword,status) VALUES ('{0}','{1}')".format(skipword, status)
			self.cur.execute(go)
			self.cur.fetchone()
			self.con.commit()
			return True
		else:
			return False
			
	def delSkip(self,skipword):
		if not skipword:
			return
		if self.existSkip(skipword):
			skipword = skipword.replace("'","''")
			go = "DELETE FROM skip WHERE skipword = '{0}'".format(skipword)
			self.cur.execute(go)
			self.cur.fetchone()
			self.con.commit()

	def existSkip(self,skipword):
		if not skipword:
			return
		skipword = skipword.replace("%","\%")
		skipword = skipword.replace("'","''")
		go = "SELECT COUNT (*) FROM skip WHERE skipword LIKE '{0}'".format(skipword)
		self.cur.execute(go)
		count = self.cur.fetchone()
		print(count)
		if count and int(count[0]) > 0:
			return True
		else:
			return False


	def setSkipStatus(self,skipword,status="False"):
		if not skipword:
			return
		skipword = skipword.replace("%","\%")
		skipword = skipword.replace("'","''")
		go = "UPDATE skip SET status == '{1}' WHERE skipword LIKE '{0}'".format(skipword, status)
		self.cur.execute(go)
		self.con.commit()



##############################	
#### whitelist statements ####
##############################

	def addToWhitelist(self,id_channel,title,description, status="True"):
		if not title:
			return
		if not description:
			description = ""
		if not id_channel:
			id_channel = 0
		
		if not self.existsInWhitelist(id_channel, title, description):
			title = title.replace("'","''")
			description = description.replace("'","''")
			go="""INSERT OR IGNORE INTO
					whitelist (id_channel,title,description,status)
					VALUES ({0},'{1}','{2}','{3}')""".format(id_channel,title, description, status)
			self.cur.execute(go)
			self.con.commit()
			return True
		else:
			return False
			
	def updateWhitelistEntry(self,id_whitelist,title):
		if not title or not id_whitelist:
			return
		title = title.replace("'","''")
		go="""UPDATE whitelist SET title = '{1}' WHERE id_whitelist == {0}""".format(id_whitelist,title)
		self.cur.execute(go)
		self.con.commit()
			
	def delFromWhitelist(self,id_channel,title,description=""):
		if not title:
			return
		title = title.replace("%","\%")
		title = title.replace("'","''")
		go = """DELETE FROM whitelist
				WHERE id_channel == {0}
				AND title LIKE '{1}'
				AND description LIKE '{2}'""".format(id_channel,title,description)
		self.cur.execute(go)
		self.con.commit()


	def delFromWhitelistById(self,id_whitelist):
		if not id_whitelist:
			return
		go = """DELETE FROM whitelist WHERE id_whitelist == {0}""".format(id_whitelist)
		self.cur.execute(go)
		self.con.commit()

	def checkWhitelist(self,id_channel,title,description,status="True",exact="True"):
		if not title or len(title) < 1:
			return
		title = title.replace("'","''")
		title = title.replace("%","\%")
		if exact is not None and exact == "False":
			go = """SELECT COUNT(id_whitelist) FROM whitelist WHERE title LIKE '%{0}%'""".format(title)
		else:
			go = """SELECT COUNT(id_whitelist) FROM whitelist WHERE title LIKE '{0}'""".format(title)
		
		if id_channel and int(id_channel) > 0:
			go += """ AND id_channel = {0} """.format(id_channel)
		if description and len(description) > 0:
			description = description.replace("'","''")
			go = """ AND description LIKE '{0}'""".format(description)
		
		go += " AND status == '{0}'".format(status)
		self.cur.execute(go)
		count = self.cur.fetchone()
		if count and int(count[0]) > 0:
			return True
		else:
			return False

			
	def existsInWhitelist(self,id_channel,title,description):
		if not title or not id_channel:
			return
		title = title.replace("%","\%")
		title = title.replace("'","''")
		if description:
			description = description.replace("'","''")
			description = description.replace("%","\%")
			go = """SELECT COUNT (*)
				FROM whitelist
				WHERE id_channel == {0}
				AND title LIKE '{1}'
				AND description LIKE '{2}'""".format(id_channel,title,description)
		else:
			go = """SELECT COUNT (*)
				FROM whitelist
				WHERE id_channel == {0}
				AND title LIKE '{1}'""".format(id_channel,title)
		self.cur.execute(go)
		count = self.cur.fetchone()
		if count and int(count[0]) > 0:
			return True
		else:
			return False

	def readWhitelist(self,status=None):
		go = "SELECT * FROM whitelist"
		if status is not None:
			go += " WHERE status == '{0}'".format(status)
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
	def getActiveWhitelistTitle(self):
		go = "SELECT title FROM whitelist WHERE status == 'True'"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows
		
	def changeWhitelistStatus(self,id_whitelist):
		if not id_whitelist:
			return
		go = "SELECT status FROM whitelist WHERE id_whitelist == {0}".format(id_whitelist) 
		self.cur.execute(go)
		data = self.cur.fetchone()

		if data[0] == "True":
			go = "UPDATE whitelist SET status == 'False' WHERE id_whitelist == {0}".format(id_whitelist)
			self.cur.execute(go)
			self.con.commit()
		else:
			go = "UPDATE whitelist SET status == 'True' WHERE id_whitelist == {0}".format(id_whitelist)
			self.cur.execute(go)
			self.con.commit()

###########################
#### addlog statements ####
###########################

	def addLog(self,timestr,msg):
		go="INSERT OR IGNORE INTO log VALUES('{0}','{1}')".format(timestr,msg)
		self.cur.execute(go)
		#self.cur.fetchone()
		self.con.commit()

	def readLog(self):
		go = "SELECT * FROM log"
		self.cur.execute(go)
		rows = self.cur.fetchall()
		return rows




##############################################
#### 'themoviedb' and 'poster' statements ####
##############################################

	#              1     1                n    1
	# [broadcast] <-------> [themoviedb] <------> [poster]
	#
	# 'themoviedb' maps between broadcast's and poster's
	# One poster can be mapped to more than one broadcast.
	# A poster is unique to its download url.

	def getMovieInfoPoster(self, pk_broadcast, m_id_movie):
		if pk_broadcast and int(pk_broadcast) > 0:
			go = """SELECT data FROM poster
					INNER JOIN themoviedb ON themoviedb.fk_poster = poster.pk_poster
					WHERE themoviedb.fk_broadcast == {}
					LIMIT 1 """.format(pk_broadcast)
		elif m_id_movie and int(m_id_movie) > 0:
			go = """SELECT data FROM poster
					INNER JOIN themoviedb ON themoviedb.fk_poster = poster.pk_poster
					WHERE m_id_movie == {}
					LIMIT 1 """.format(m_id_movie)
		else:
			return None
		self.cur.execute(go)
		data = self.cur.fetchone()
		if data:
			return data[0]
		else:
			return None
			
	def addPoster(self, poster_url, data):
		if not poster_url or not data:
			return None
		binary = sqlite3.Binary(data)
		self.cur.execute("INSERT OR IGNORE INTO poster (url, data) VALUES (?,?)", [poster_url, binary])
		self.con.commit()
		go = "SELECT MAX(pk_poster) FROM poster"
		self.cur.execute(go)
		pk_posters = self.cur.fetchone()
		if not pk_posters:
			return None
		return int(pk_posters[0])

	def existsPoster(self, poster_url):
		if poster_url and poster_url != "":
			poster_url = poster_url.replace("'", "''")
			go = """SELECT pk_poster FROM poster WHERE url == '{}' """.format(poster_url)
		else:
			return None
		self.cur.execute(go)
		pk_posters = self.cur.fetchone()
		if pk_posters and int(pk_posters[0]) > 0:
			return int(pk_posters[0])
		else:
			return None
	
	def addUpdateNewMovieInfo(self, pk_themoviedb, pk_broadcast, pk_poster, m_id_movie, m_name, m_year, m_title_org, m_rating, m_description, m_genre):
		if not m_name or not m_id_movie:
			return False
		if not m_description:
			m_description = ""
		if not pk_broadcast:
			return False
		if m_genre:
			# its not cool, but we need to store our genres as string
			m_genre = ','.join(str(x) for x in m_genre if x)
		else:
			m_genre = ""

		m_genre = m_genre.replace("'", "''")
		m_title_org = m_title_org.replace("'", "''")
		m_name = m_name.replace("'", "''")
		m_description = m_description.replace("'", "''")

		if not pk_themoviedb:
			go="""INSERT OR IGNORE INTO
					themoviedb (fk_broadcast, fk_poster, m_id_movie, m_name, m_year, m_title_org, m_rating, m_description, m_genre)
					VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?) """
			self.cur.execute(go, (pk_broadcast, pk_poster, m_id_movie, m_name, m_year, m_title_org, m_rating, m_description, m_genre))
			self.con.commit()
		else:
			go = """UPDATE themoviedb
					SET fk_broadcast = {}, 
						fk_poster = {}, 
						m_id_movie = {}, 
						m_name = '{}', 
						m_year = '{}', 
						m_title_org = '{}', 
						m_rating = '{}', 
						m_description = '{}', 
						m_genre = '{}' 
					WHERE pk_themoviedb == {}
					""".format(pk_broadcast, pk_poster, m_id_movie, m_name, m_year, m_title_org, m_rating, m_description, m_genre, pk_themoviedb)
			self.cur.execute(go)
			self.con.commit()
		return True


	def updateMovieInfoPoster(self, m_id_movie, pk_poster):
		go = """UPDATE themoviedb SET fk_poster == {} WHERE m_id_movie == {} """.format(pk_poster, m_id_movie)
		self.cur.execute(go)
		self.con.commit()


	def existsInTheMovieDB(self, pk_broadcast):
		#if not m_id_movie:
		#	return False
		if pk_broadcast and int(pk_broadcast) > 0:
			go = """SELECT pk_themoviedb FROM themoviedb WHERE fk_broadcast == {} """.format(pk_broadcast)
		else:
			return None
		go += " ORDER BY pk_themoviedb DESC LIMIT 1"
		self.cur.execute(go)
		pk_themoviedb = self.cur.fetchone()
		if pk_themoviedb:
			return int(pk_themoviedb[0])
		else:
			return None

	def resetMovieInfoActive(self, pk_broadcast=None, m_id_movie=None):
		if pk_broadcast and int(pk_broadcast) > 0:
			go = """DELETE FROM themoviedb WHERE fk_broadcast == {}""".format(pk_broadcast)
		elif m_id_movie and int(m_id_movie) > 0:
			go = """DELETE FROM themoviedb WHERE m_id_movie == {}""".format(m_id_movie)
		else:
			return
		#with open("/tmp/skyrecorder_querydebug.txt" , "w") as f:
		#	f.write(str(go))
		self.cur.execute(go)
		self.con.commit()

	def getMovieInfo(self, pk_broadcast, m_id_movie):
		if pk_broadcast and int(pk_broadcast) > 0:
			go = """SELECT * FROM themoviedb WHERE fk_broadcast == {} LIMIT 1 """.format(pk_broadcast)
		elif m_id_movie and int(m_id_movie) > 0:
			go = """SELECT * FROM themoviedb WHERE m_id_movie == {} LIMIT 1 """.format(m_id_movie)
		else:
			return []
		self.cur.execute(go)
		rows = self.cur.fetchall()
		if rows and len(rows) > 0:
			return rows
		else:
			return []
	
	def getMovieInfoMovieId(self, pk_broadcast):
		if pk_broadcast and int(pk_broadcast) > 0:
			go = """SELECT m_id_movie FROM themoviedb WHERE fk_broadcast == {} LIMIT 1 """.format(pk_broadcast)
		else:
			return None
		self.cur.execute(go)
		m_id_movie = self.cur.fetchone()
		if m_id_movie and int(m_id_movie[0]) > 0:
			return int(m_id_movie[0])
		else:
			return None
			
	def cleanupTMDbData(self, inactive=True):
		if inactive:
			# delete inactive and unused "mapping" entries from themoviedb
			go = """DELETE FROM themoviedb 
					WHERE themoviedb.fk_broadcast NOT IN (SELECT broadcast.pk_broadcast FROM broadcast)
						and themoviedb.fk_broadcast != -1 """  # manual covers from Archiv
			self.cur.execute(go)
			self.con.commit()
			# delete unused posters
			go = """DELETE FROM poster 
					WHERE poster.pk_poster NOT IN (SELECT themoviedb.fk_poster FROM themoviedb)
						AND poster.pk_poster NOT IN (SELECT broadcast.fk_poster FROM broadcast) """
			self.cur.execute(go)
			self.con.commit()
		else:
			go = "DELETE FROM poster"
			self.cur.execute(go)
			self.con.commit()
			go = "DELETE FROM themoviedb"
			self.cur.execute(go)
			self.con.commit()

#############################
#### database statements ####
#############################
	
	def shrinkDatabase(self):
		self.con.execute("VACUUM")
	
	def truncateDatabase(self, includeAdded=False, includeMovieInfo=False):
		
		try:
			#go = "DELETE FROM eventdetails"
			#self.cur.execute(go)
			#self.con.commit()
			go = "DELETE FROM event"
			self.cur.execute(go)
			self.con.commit()
			#go = "DELETE FROM eventcover"
			#self.cur.execute(go)
			#self.con.commit()
			go = "DELETE FROM broadcast"
			self.cur.execute(go)
			self.con.commit()
			if includeAdded:
				go = "DELETE FROM added"
				self.cur.execute(go)
				self.con.commit()
			if includeMovieInfo:
				go = "DELETE FROM themoviedb"
				self.cur.execute(go)
				self.con.commit()
				go = "DELETE FROM poster"
				self.cur.execute(go)
				self.con.commit()
		except Exception as e:
			print('Exception in truncateDatabase: {}'.format(e))
			return False
		return True

# get instance	
sql = sqlcmds()
