#-*- coding: utf-8 -*-
import sys, urllib, re, os, time
#import socket
import json
import requests
import codecs

from Components.config import config, configfile

from .SkySql import *
from .SkyMainFunctions import *

class SkyTheMovieDB(object):
	
	def __init__(self,timeout=10):
		self.alternate_lang = "en"
		self.timeout = timeout
		try:
			self.sky_log_path = config.plugins.skyrecorder.skylog.value
		except:
			self.sky_log_path = "/usr/lib/enigma2/python/Plugins/Extensions/skyrecorder/sky_log"
		if not os.path.isfile(self.sky_log_path):
			file = codecs.open(self.sky_log_path, "w", "utf-8")
			file.write('\xef\xbb\xbf')
			file.close()

		self.headersJSON = {
			'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/20100101 Firefox/19.0',
			'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
			'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
			'Cache-Control': 'max-age=3',
			'Accept-Encoding': 'gzip, deflate',
			'Accept-Language': 'de-de,de;q=0.8,en-us;q=0.5,en;q=0.3'}
		self.headersImage = getHttpHeader2()

	def cleanName(self,text):
		#text1 = text.decode("iso-8859-1")
		#text = text1.encode("utf-8")
		text = text.replace('&amp;','&')
		text = text.replace('/','')
		text = text.replace(':','_') # for enigma2 getRecordingFilename
		text = text.replace('–','-')
		# finally kill the rest
		text = re.sub('-+',"-",text)
		return text
	
	def prepareSearchString(self,text):
		#text1 = text.decode("iso-8859-1")
		#text = text1.encode("utf-8")
		
		# FIXME: some hacks
		text = re.sub("[Mm]arvel's ","",text)
		text = re.sub("[?]","",text)
		try:  # Python 2
			text = urllib.quote_plus(text)
		except:  # Python 3
			text = urllib.parse.quote_plus(text)
		return text
		
		text = text.replace('&auml;','ae').replace('&Auml;','Ae').replace('&ouml;','oe').replace('&ouml;','Oe').replace('&uuml;','ue')
		text = text.replace('&Uuml;','Ue').replace('&szlig;','ss').replace('&amp;','&').replace('&quot;','\"').replace('&gt;','\'')
		text = text.replace('&#228;','ae').replace('&#246;','oe').replace('&#252;','ue').replace('&#223;','ss').replace('&#8230;','...')
		text = text.replace('\\u00c4','Ae').replace('\\u00e4;',"ae").replace('\\u00d6',"Oe").replace('\\u00f6','oe')
		text = text.replace('\\u00dc','Ue').replace('\\u00fc',"ue").replace('\\u00df',"ss")
		text = text.replace('&#196;','Ae').replace('&#228;',"ae").replace('&#214;',"Oe").replace('&#246;',"oe").replace('&#220;',"Ue").replace('&#252;',"ue")
		text = text.replace('ö','oe').replace('ü','ue').replace('ä','ae').replace('Ã','ss')
		text = text.replace('Ö','Oe').replace('Ü','Ue').replace('Ä','Ae')
		text = text.replace('–','-')
		text = text.replace('/','-')
		text = text.replace(' ','-')
		# finally kill the rest
		text = re.sub("[^a-zA-Z0-9-]","",text)
		text = re.sub('-+',"-",text)
		return text
	
	
	def getListFor(self, searchStr, is_series=False, language="de"):
		"""
		:return: 	result[][] <br>
					result[n]["m_movie_url"] = str <br>
					result[n]["m_title_org"] = str <br>
					result[n]["m_title"] = str <br>
					result[n]["m_poster_url"] = str <br>
					result[n]["m_date"] = str <br>
					result[n]["m_id_movie"] = str <br>
					result[n]["m_description"] = str <br>
					result[n]["m_rating"] = str <br>
					result[n]["m_genre"] = [] <br>
		"""
		if not searchStr or len(searchStr) < 1:
			return False
		
		searchstring = self.prepareSearchString(searchStr)
		if is_series:
			url = "http://api.themoviedb.org/3/search/tv?api_key={0}&query={1}&language={2}".format(config.plugins.skyrecorder.apikey.value, searchstring, language)
		else:
			url = "http://api.themoviedb.org/3/search/movie?api_key={0}&query={1}&language={2}".format(config.plugins.skyrecorder.apikey.value,searchstring, language)
		self.addLog("Durchsuche TMDB nach: {0}".format(self.hideApiKey(url)))
		print("Durchsuche TMDB nach: {0}".format(self.hideApiKey(url)))

		movie_details = self.doRequestJSON(url)
		if not movie_details:
			return False

		# store our results
		resultList = []

		parsed_json = json.loads(movie_details)
		#self.addLog("Ergebnis: {0}".format(parsed_json))

		for result in parsed_json["results"]:
			# same parameters for movies and series'
			m_id_movie = str(result["id"])
			m_genre = [].append(genre['name'] for genre in (result.get('genres', None) or []))
			m_rating = str(result.get("vote_average", "0"))

			#################### movie ####################
			if not is_series:
				if not result["poster_path"] or len(result["poster_path"]) < 6:
					continue  # no movie poster
				m_poster_url = str("http://image.tmdb.org/t/p/w300_and_h450_bestv2{0}".format(result["poster_path"]))
				m_title_org = str(result.get("original_title", None) or "")
				m_title = str(result.get("title", None) or "")
				m_date = str(result.get("release_date", None) or "")[0:4]
				m_description = str(result.get("overview", None) or "")
				m_movie_url = str("http://api.themoviedb.org/3/movie/{0}?api_key={1}&language={2}".format(result["id"], config.plugins.skyrecorder.apikey.value, language))
				resultList.append({	"m_movie_url": 		m_movie_url,
									"m_title_org": 		m_title_org,
									"m_title": 			m_title,
									"m_poster_url": 	m_poster_url,
									"m_date": 			m_date,
									"m_id_movie": 		m_id_movie,
									"m_description":	m_description,
									"m_rating":			m_rating,
									"m_genre":			m_genre		})
			#################### series ####################
			else:
				series_name_org = str(result.get("original_name", None) or "")
				series_name = str(result.get("name", None) or "")
				series_details_url = str("http://api.themoviedb.org/3/tv/{0}?api_key={1}&language={2}".format(result["id"], config.plugins.skyrecorder.apikey.value, language))
				series_description = str(result.get("overview", None) or "")
				if result["poster_path"] and len(result["poster_path"]) > 5:
					series_poster_url = str("http://image.tmdb.org/t/p/w300_and_h450_bestv2{0}".format(result["poster_path"]))
					series_date = str(result.get("first_air_date", None) or "")[0:4]
					resultList.append({	"m_movie_url": 		series_details_url,
										"m_title_org": 		series_name_org,
										"m_title": 			series_name,
										"m_poster_url":		series_poster_url,
										"m_date": 			series_date,
										"m_id_movie": 		m_id_movie,
										"m_description":	series_description,
										"m_rating":			m_rating,
										"m_genre":			m_genre	})

				# also check for season posters and season's "air_date"
				#################### seasons ####################
				series_details = self.doRequestJSON(series_details_url)
				if series_details:
					details_json = json.loads(series_details)
					for season in (details_json.get('seasons', None) or []):
						try:
							if season.get('season_number', 0) > 0:  # season 0 are extras
								season_poster = season.get('poster_path', None)
								if season_poster and len(season_poster) > 5:
									season_id = season.get('id', None) or m_id_movie
									season_date = str(season.get('air_date', None) or "")[0:4]
									season_poster_url = str("http://image.tmdb.org/t/p/w300_and_h450_bestv2{0}".format(season_poster))
									season_description = str(season.get('overview', None))
									if not season_description or len(season_description) == 0:
										season_description = series_description
									resultList.append({	"m_movie_url": 		series_details_url,
														"m_title_org": 		series_name_org,
														"m_title": 			series_name,
														"m_poster_url": 	season_poster_url,
														"m_date": 			season_date,
														"m_id_movie": 		season_id,
														"m_description":	season_description,
														"m_rating":			m_rating,
														"m_genre":			m_genre	})
						except Exception as e:
							print('Exception while checking season of a series: {}'.format(e))

		return resultList


	def doRequestJSON(self, url):
		try:
			response = requests.request('GET', url, headers=self.headersJSON, timeout=self.timeout)
			if response.status_code != 200:
				self.addLog('Could not download TMDb info: error={0} url={1}'.format(response.status_code, url))
				print('Could not download TMDb info: error={0} url={1}'.format(response.status_code, url))
				return None
			data = response.content
			if data and len(data) > 0:
				return data
			else:
				self.addLog('No response from TMDb web site: url={}'.format(url))
				print('No response from TMDb web site: url={}'.format(url))
				return None
		except Exception as e:
			self.addLog('Could not search for TMDb info. Exception: {0}'.format(e))
			print('Could not search for TMDb info. Exception: {0}'.format(e))
			return None


	def getInfoFor(self, movieURL="", movieTitle="", idMovie="", language="de"):
		if movieURL and len(movieURL) > 1:
			url = "{0}&language={1}".format(movieURL,language)
			#print(self.hideApiKey(url))
			self.addLog("Abfrage Detailinfos TMDB: {0}".format(self.hideApiKey(url)))
		else:
			return False

		data = self.doRequestJSON(movieURL)
		if not data:
			print("nothing found")
			return False

		resultList = []
				
		parsed_json = json.loads(data)

		try:
			m_name = str(parsed_json["title"])
		except:
			m_name = str(parsed_json["name"])  # for series
		try:
			m_title_org = str(parsed_json["original_title"])
		except:
			m_title_org = str(parsed_json["original_name"])  # for series
		try:
			m_year = str(parsed_json["release_date"])[0:4]
		except:
			m_year = str(parsed_json.get("first_air_date", None) or "")[0:4]  # for series
		m_rating = str(parsed_json["vote_average"])
		m_description = str(parsed_json["overview"])
		m_posters = str("http://image.tmdb.org/t/p/w300_and_h450_bestv2{0}".format(parsed_json["poster_path"]))
		m_id_movie = str(parsed_json["id"])
		m_genre = []
		for m_results2 in parsed_json["genres"]:
			treffer = json.loads(json.dumps(m_results2))
			m_genre.append(str(treffer["name"]))
				
		resultList.append({	"m_name":			m_name,
							"m_year":			m_year,
							"m_title_org":		m_title_org,
							"m_rating":			m_rating,
							"m_description":	m_description,
							"m_posters":		m_posters,
							"m_id_movie":		m_id_movie,
							"m_genre":			m_genre		})
							
		return resultList

	def RunAutoloadInfosTMDB(self):
		self.addLog("RunAutoloadInfos")
		if config.plugins.skyrecorder.apikey.value == "00000000000000000000000000000000":
			self.addLog("No TMDb api key, no search")
			return
		movies = sql.getEventsMainAutoCheck_(order="ASC")
		if len(movies) > 0:
			#self.addLog("Movie-Eintraege gefunden")
			for movie in movies:
				# get TMDB infos if no infos in database
				if int(movie[14]) == 0:
					#self.addLog("Keine DB-Eintraege fuer Event {0} gefunden".format(movie[0]))
					pk_broadcast = int(movie[0])
					title = str(movie[1])
					year = str(movie[15])
					is_series = movie[16] and movie[16] > 0

					self.doMovieInfoAutocheck(title=title, is_series=is_series, year=year, pk_broadcast=pk_broadcast)


	def searchMovieInfos(self, title, is_series=False, language="de"):
		"""
		:return: 	result[][] <br>
					result[n]["m_movie_url"] = str <br>
					result[n]["m_title_org"] = str <br>
					result[n]["m_title"] = str <br>
					result[n]["m_poster_url"] = str <br>
					result[n]["m_date"] = str <br>
					result[n]["m_id_movie"] = str <br>
					result[n]["m_description"] = str <br>
					result[n]["m_rating"] = str <br>
					result[n]["m_genre"] = [] <br>
		"""
		try:
			results = self.getListFor(searchStr=title, is_series=is_series, language=language)
			if not results or len(results) < 1:
				for separator in ['[', '(', '-', ':', '...', '/', '{', '_', '~', '*', '#', '^']:
					if separator in title:
						searchStr = title.split(separator)[0].strip()
						results = self.getListFor(searchStr=searchStr, is_series=is_series, language=language)
						if results and len(results) > 0:
							break

			if not results or len(results) < 1:
				results = []
		except Exception as e:
			self.addLog("searchMovieInfos error: {0}".format(e))
			print("searchMovieInfos error: {0}".format(e))
			results = []
		return results


	def doMovieInfoAutocheck(self, title, is_series=False, year=None, language="de", pk_broadcast=0):
		self.addLog("TMDb search for '{}' is_series={}".format(title, is_series))
		fixedTitle = title
		# for an exact match, cut off endings with [, ( or { e.g. "(Director's Cut)" or "[Uncut]"
		for separator in ['[', '(', '{']:
			if separator in fixedTitle:
				fixedTitle = fixedTitle.split(separator)[0].strip()
		results = self.searchMovieInfos(fixedTitle, is_series=is_series, language=language)
		# go through filmlist and get entry with newest date
		actualYear = time.localtime().tm_year
		latestYear = 0
		matching_result = None
		for onlyExactYear in [True, False]:
			if matching_result:
				break  # if had an exact match by year and title in first loop
			for result in results:
				movieTitle = str(result["m_title"])
				m_date = result["m_date"]
				if not m_date or len(m_date) != 4:
					continue  # no year in tmdb result
				tmdbYear = int(m_date)
				#print('############################# skyTitle="{0}", skyYear={1}, tmdbYear={2}, tmdbTitle="{3}"'.format(title, year, tmdbYear, str(result["m_title"])))

				if onlyExactYear:  # first main loop
					if year and int(year) == tmdbYear and fixedTitle == movieTitle:
						matching_result = result
						break  # exact year and title
				else:  # second main loop (if necessary)
					# if we have a movie year in tv guide, a cover has to be in a range of +- 3 years
					if year and abs(tmdbYear - int(year)) > 3:
						#print('############################# OUT OF RANGE')
						continue  # not in range of +- 3 years
					# no covers from the future
					if tmdbYear > actualYear:
						#print('############################# IN THE FUTURE')
						continue  # in the future
					#print('############################# OK')
					if tmdbYear > latestYear or fixedTitle == movieTitle:
						latestYear = tmdbYear
						matching_result = result
						if fixedTitle == movieTitle:
							#print('############################# TITLE MATCH')
							break  # exact title match (and year in range), we can break

		if matching_result:  # the TMDb auto check does not update existing covers
			self.storeMovieInfoAndCoverToDatabase(pk_broadcast=pk_broadcast, sky_title=title, result=matching_result, updateAllowed=False)


	def storeMovieInfoAndCoverToDatabase(self, pk_broadcast=0, sky_title="", result=None, updateAllowed=False):

		# result["m_movie_url"] 	= str
		# result["m_title_org"] 	= str
		# result["m_title"] 		= str
		# result["m_poster_url"] 	= str
		# result["m_date"] 			= str
		# result["m_id_movie"] 		= str
		# result["m_description"] 	= str
		# result["m_rating"] 		= str
		# result["m_genre"] 		= []

		m_name = result['m_title']
		if m_name == "":
			m_name = None
			
		if result['m_id_movie'] and result['m_id_movie'] != "":
			m_id_movie = int(result['m_id_movie'])
		else:
			m_id_movie = None
			
		m_title_org = result['m_title_org']
		if not m_title_org:
			m_title_org = ""

		m_year = result['m_date']
		if not m_year:
			m_year = ""

		m_description = result['m_description']
		if not m_description:
			m_description = ""
			
		m_genre = result['m_genre']
		if not m_genre:
			m_genre = []

		m_rating = result['m_rating']
		if not m_rating:
			m_rating = "0"

		m_poster_url = result["m_poster_url"]

		pk_themoviedb = sql.existsInTheMovieDB(pk_broadcast)

		if pk_themoviedb and not updateAllowed:
			return

		pk_poster = sql.existsPoster(m_poster_url)

		if not pk_poster:
			try:
				response = requests.request('GET', m_poster_url, headers=self.headersImage, timeout=10)
				if 200 != response.status_code:
					self.addLog('TMDB: Could not download cover. Http error: {0}'.format(response.status_code))
					return
				data = response.content
			except Exception as e:
				self.addLog('TMDB: Could not download cover. Exception: {0}'.format(e))
				return

			time.sleep(0.123)
			if data and len(data) > 0:
				try:
					pk_poster = sql.addPoster(m_poster_url, data)
					self.addLog('TMDb cover downloaded and stored in db: {0}'.format(m_poster_url))
					print('TMDb cover downloaded and stored in db: {0}'.format(m_poster_url))
				except Exception as e:
					self.addLog('TMDb: Could not store cover to db. Exception: {0}'.format(e))
					return

		if pk_poster:
			if sql.addUpdateNewMovieInfo(pk_themoviedb, pk_broadcast, pk_poster, m_id_movie, m_name, m_year, m_title_org, m_rating, m_description, m_genre):
				self.addLog("Info hinzugefuegt fuer: {0}".format(sky_title))
				print("Info hinzugefuegt fuer: {0}".format(sky_title))


	def addLog(self, text):
		if len(text) < 1:
			return
		# check the current file size truncate the file if size is greater than defined limit 200 KB (204800 Bytes)
		sizeb = os.path.getsize(self.sky_log_path)
		if sizeb > 204800:
			# truncate only the first 100 lines in file - delete the oldest ones
			with open(self.sky_log_path, "r+") as f:
				for x in range(100):
					f.readline()
					f.truncate()

		lt = time.localtime()
		datum = time.strftime("%d.%m.%Y - %H:%M:%S", lt)
		with open(self.sky_log_path , "a") as write_log:
			write_log.write('"%s - %s"\n' % (datum,text))

	def hideApiKey(self, url):
		if ("00000000000000000000000000000000" != config.plugins.skyrecorder.apikey.value):
			return url.replace(config.plugins.skyrecorder.apikey.value, "********************************")
		return url
