#-*- coding: utf-8 -*-
from Components.UsageConfig import preferredTimerPath, preferredInstantRecordPath
from Components.config import config
from Screens.MessageBox import MessageBox
import os, sys

# Navigation (RecordTimer)
import NavigationInstance

# Timer
from ServiceReference import ServiceReference
from RecordTimer import RecordTimerEntry
from RecordTimer import RecordTimer, parseEvent, AFTEREVENT

from Components.TimerSanityCheck import TimerSanityCheck
from .SkyMainFunctions import getChannelByRef, getRecordFilename, buildSkyChannellist


class SkyTimerRec():

	#def __init__(self, session):
	#	self.session = session
	
	
	@staticmethod	
	def getTimersList():
	
		recordHandler = NavigationInstance.instance.RecordTimer
		
		entry = None
		timers = []
		sky_chlist = buildSkyChannellist()
		
		for timer in recordHandler.timer_list:
			if timer and timer.service_ref and timer.eit is not None:

				location = 'NULL'
				channel = 'NULL'
				recordedfile ='NULL' 
				if timer.dirname:
					location = timer.dirname
				channel = getChannelByRef(sky_chlist,str(timer.service_ref))
				if channel:
					recordedfile = getRecordFilename(timer.name,timer.description,timer.begin,channel)
				timers.append({
					"title": timer.name,
					"description": timer.description,
					"id_channel": 'NULL',
					"channel": channel,
					"id_genre": 'NULL',
					"begin": timer.begin,
					"end": timer.end,
					"serviceref": timer.service_ref,
					"location": location,
					"recordedfile": recordedfile,
					"tags": timer.tags,
					"eit" : timer.eit
				})
		return timers		


	@staticmethod
	def getTimerEntry(name, description, timer_starttime, channelref):

		recordHandler = NavigationInstance.instance.RecordTimer

		for timer in recordHandler.timer_list:
			if timer and timer.service_ref \
					and str(timer.name) == name \
					and str(timer.description) == description \
					and int(timer.begin) == timer_starttime \
					and str(timer.service_ref) == channelref:
				return timer

		return None


	@staticmethod
	def removeTimerEntry(timer):
		recordHandler = NavigationInstance.instance.RecordTimer
		recordHandler.removeEntry(timer)

		
	@staticmethod	
	def addTimer(session, serviceref, begin, end, name, description, disabled, justplay, afterevent, dirname, tags, repeated, logentries=None, eit=0, recordfile=None):
	
		recordHandler = NavigationInstance.instance.RecordTimer
		
		msgTimeout = config.plugins.skyrecorder.msgtimeout.value

		if not dirname:
			try:
				dirname = config.plugins.skyrecorder.anytimefolder.value
			except Exception:
				dirname = preferredTimerPath()

		try:
			try:
				timer = RecordTimerEntry(
					ServiceReference(serviceref),
					begin,
					end,
					name,
					description,
					eit,
					disabled,
					0,
					afterevent,
					dirname=dirname,
					tags=tags,
					justremind=justplay)
			#except AttributeError:
			except Exception:
				#sys.exc_clear()
				
				timer = RecordTimerEntry(
					ServiceReference(serviceref),
					begin,
					end,
					name,
					description,
					eit,
					disabled,
					justplay,
					afterevent,
					dirname=dirname,
					tags=tags)
			
	
			timer.repeated = repeated
			
			# try to overwrite the calculated filename from function calculateFilename
			# it's a hack, but it works
			#if recordfile:
			#	def getMyFileName():
			#		timer.Filename = recordfile
			#	
			#	timer.calculateFilename = getMyFileName
			
			if logentries:
				timer.log_entries = logentries

			conflicts = recordHandler.record(timer)
			if conflicts:
				errors = []
				for conflict in conflicts:
					errors.append(conflict.name)
	
				return {
					"result": False,
					"message": "Conflicting Timer(s) detected! %s" % " / ".join(errors)
				}
		except Exception as e:
			print(e)
			return {
				"result": False,
				"message": "Could not add timer '%s'!" % e
			}
			
		print("[skyrecorder] timer added.")
		if not config.plugins.skyrecorder.silent_timer_mode or config.plugins.skyrecorder.silent_timer_mode.value == False:
			message = session.open(MessageBox, _("%s - %s added.\nZiel: %s") % (name, description, dirname), MessageBox.TYPE_INFO, timeout=msgTimeout)
		
		return {
			"result": True,
			"message": "Timer '%s' added" % name,
			"eit" : eit
		}


	@staticmethod
	def getRecFilenameOfTimer(timer):
		"""
		:rtype: str
		:param timer:	the RecordTimerEntry
		:return:		path of record file without suffix .ts
		"""
		recordfile = None
		try:
			timer.calculateFilename()
			recordfile = timer.Filename
			if timer.dirname not in recordfile:
				# what the hack is going on?
				# replace the path with timer.dirname
				recordfile = timer.dirname + recordfile[recordfile.rindex('/')+1:]
		except Exception as e:
			print('Exception in getRecordfileOfTimer: {0}'.format(e))
		return recordfile


	@staticmethod
	def copyTMDbCoverToTimerRecordfile(timer, pk_broadcast=None, m_id_movie=None):
		from .SkySql import sql

		recordfile = SkyTimerRec.getRecFilenameOfTimer(timer)
		if recordfile:
			coverFilename = recordfile + '.jpg'
			#print('########### Have to add cover at "%s"' % coverFilename)

			if not os.path.exists(coverFilename):
				#print('################# Cover does not exist: %s' % coverFilename)
				cover = sql.getMovieInfoPoster(pk_broadcast, m_id_movie)
				if cover:
					print('[skyrecorder] Copy cover to: %s' % coverFilename)
					with open(coverFilename, 'wb') as f:
						f.write(cover)
					#print('################# Copy finished')
					return True
				else:
					pass  #print('################# No cover found in DB for m_id_movie: {0}!'.format(m_id_movie))
			else:
				pass  #print('################# Cover exists: %s' % coverFilename)

		return False


	@staticmethod
	def deleteTMDbCoverOfTimerRecordfile(recTimer):

		recFilename = SkyTimerRec.getRecFilenameOfTimer(recTimer)
		if recFilename:
			coverFilename = recFilename + '.jpg'
			#print('########### Have to delete cover: %s' % coverFilename)
			if os.path.exists(coverFilename):
				print('[skyrecorder] Delete cover: %s' % coverFilename)
				os.remove(coverFilename)
				#print('########### Cover deleted: %s' % coverFilename)
				return True
			else:
				pass  #print('########### Cover does not exist: %s' % coverFilename)
		else:
			pass  #print('########### No recordfile for timer: {0}'.format(recTimer))

		return False
